# This file is in the public domain.
#
# --
#
# This file is derived from the bsd.pkg.mk of NetBSD, and came with this
# CVS ID:
#	$NetBSD: bsd.pkg.mk,v 1.1223 2003/07/24 22:27:16 dmcmahill Exp $
# Many features have been removed, because they are not needed for
# GODI.
#
# --
#
# This file is derived from bsd.port.mk - 940820 Jordan K. Hubbard.
# ----------------------------------------------------------------------

# Default sequence for "all" is:  fetch checksum extract patch configure build
#
# Please read the comments in the targets section below, you
# should be able to use the pre-* or post-* targets/scripts
# (which are available for every stage except checksum) or
# override the do-* targets to do pretty much anything you want.
#
# NEVER override the "regular" targets unless you want to open
# a major can of worms.

##### Include any preferences, if not already included, and common definitions
.include "${.PARSEDIR}/bsd.prefs.mk"

# This include sets:
# - PKGDIR
# - WRKDIR

######################################################################
# PACKAGE FIXUP
######################################################################

.if !defined(CYGWIN_ROOT) 
.if defined(W32PORT) && ${W32PORT} == "mingw"
ROOT_PATH=/
CYGWIN_ROOT=${ROOT_PATH:n}
.else
CYGWIN_ROOT=
.endif
.endif

.if !defined(DISTNAME)
PKG_FAIL_REASON="Missing package property: DISTNAME"
.endif

.if !defined(CATEGORIES)
PKG_FAIL_REASON="Missing package property: CATEGORIES"
.endif

.if defined(GODI_PATH)
PKG_FAIL_REASON+='Please unset GODI_PATH before doing pkgsrc work!'
.endif


PKGBASE?=		${PKGNAME:C/-[^-]*$//}
PKGVERSION?=		${PKGNAME:C/^.*-//}
PKGWILDCARD?=		${PKGBASE}-*

# Derived names so that they're easily overridable.
DISTFILES?=		${DISTNAME}${EXTRACT_SUFX}
.if defined(PKGREVISION) && ${PKGREVISION} != "" && ${PKGREVISION} != "0"
.  if defined(PKGNAME)
PKGNAME_NOREV:=		${PKGNAME}
PKGNAME:=		${PKGNAME}godi${PKGREVISION}
.  else
PKGNAME?=		${DISTNAME}godi${PKGREVISION}
PKGNAME_NOREV=		${DISTNAME}
.  endif
.else
PKGNAME?=		${DISTNAME}
PKGNAME_NOREV=		${PKGNAME}
.endif
SVR4_PKGNAME?=		${PKGNAME}

MAINTAINER?=		nobody
DISTPREFIX?=
DISTPREFIX_IN_DISTINFO?=${DISTPREFIX}
# you can have a different prefix in the distinfo file

.if !defined(COMMENT)
COMMENT!=	(${CAT} ${PKGDIR:Q}/COMMENT || ${ECHO} -n "(no description)") 2>/dev/null
.endif

WRKSRC?=		${WRKDIR}/${DISTNAME}

DESCR=			${WRKDIR}/.DESCR
DESCR_SRC?=		${PKGDIR}/DESCR
PLIST=			${WRKDIR}/.PLIST
PLIST_SRC?=		${PKGDIR}/PLIST
FLIST=			${WRKDIR}/.FLIST

# By setting GODI_PLIST, the file PLIST.godi is processed instead of
# PLIST. This is now documented in project_doc/plist.ref.txt.

.if defined(GODI_PLIST)
PLIST_GODI_FILE?=PLIST.godi
.if ${PLIST_GODI_FILE:M/*} == ""
PLIST_SRC=${PKGDIR}/${PLIST_GODI_FILE}
.else
PLIST_SRC=${PLIST_GODI_FILE}
.endif
.endif

# Set INSTALL_FILE to be the name of any INSTALL file
.if !defined(INSTALL_FILE) && exists(${PKGDIR}/INSTALL)
INSTALL_FILE=		${PKGDIR}/INSTALL
.endif

# Set DEINSTALL_FILE to be the name of any DEINSTALL file
.if !defined(DEINSTALL_FILE) && exists(${PKGDIR}/DEINSTALL)
DEINSTALL_FILE=		${PKGDIR}/DEINSTALL
.endif

# If MESSAGE hasn't been defined, then set MESSAGE_SRC to be a space-separated
# list of files to be concatenated together to generate the MESSAGE file.
#
.if !defined(MESSAGE_SRC) && !defined(MESSAGE)
.  if exists(${PKGDIR}/MESSAGE)
MESSAGE_SRC=		${PKGDIR}/MESSAGE
.  endif
.endif

# If this is empty, then everything gets extracted.
EXTRACT_ELEMENTS?=	# empty
BUILD_DIRS?=		${WRKSRC}
CONFIGURE_DIRS?=	${WRKSRC}
TEST_DIRS?=		${BUILD_DIRS}
INSTALL_DIRS?=		${BUILD_DIRS}

######################################################################
# GMAKE/OMAKE
######################################################################

.if defined(USE_GMAKE)

# GODI: We currently do not have gmake in the "base" category, but if we
# had it, the following addition would make sense:
#
#.  if ${_OPSYS_HAS_GMAKE} == "no"
#BUILD_DEPENDS+=		gmake>=3.78:../../devel/gmake
#.  endif

MAKE_PROGRAM=		${GMAKE}
.else
.if defined(USE_OMAKE)
MAKE_PROGRAM?=		${LOCALBASE}/bin/omake
BUILD_DEPENDS+=	        godi-omake>=0:../../godi/godi-omake
.else
.if defined(CONF_SCRIPT) || defined(CONF_SCRIPT_TEXT)
MAKE_PROGRAM?=		${TRUE}
.else
#previously this was ${MAKE}, USE_GMAKE is obsolete now
MAKE_PROGRAM?=		${GMAKE}
.endif
.endif
.endif
CONFIGURE_ENV+=		MAKE=${MAKE_PROGRAM:Q}

MAKE_FLAGS?=
MAKEFILE?=		Makefile

######################################################################
# ULIMIT
######################################################################

# Automatically increase process limit where necessary for building.
_ULIMIT_CMD=
.if defined(UNLIMIT_RESOURCES)
.  for __tmp__ in ${UNLIMIT_RESOURCES}
.    if defined(ULIMIT_CMD_${__tmp__})
_ULIMIT_CMD+=	${ULIMIT_CMD_${__tmp__}} ;
.    endif
.  endfor
.endif

######################################################################
# TOOLCHAIN CONFIG
######################################################################

.if ${ELF_RPATH} == "yes"
LDFLAGS+=		-Wl,${ELF_RPATH_FLAG}${LOCALBASE}/lib
.endif
LDFLAGS+=		-L${LOCALBASE}/lib

######################################################################
# COOKIES, INTERNAL CONFIG
######################################################################

EXTRACT_COOKIE=		${WRKDIR}/.extract_done
BUILDLINK_COOKIE=	${WRKDIR}/.buildlink_done
CONFIGURE_COOKIE=	${WRKDIR}/.configure_done
INSTALL_COOKIE=		${WRKDIR}/.install_done
TEST_COOKIE=		${WRKDIR}/.test_done
BUILD_COOKIE=		${WRKDIR}/.build_done
PATCH_COOKIE=		${WRKDIR}/.patch_done
PACKAGE_COOKIE=		${WRKDIR}/.package_done
INTERACTIVE_COOKIE=	.interactive_stage
NULL_COOKIE=		${WRKDIR}/.null

# New message digest defs
DIGEST_ALGORITHM?=	SHA1

# Miscellaneous overridable commands:
SHCOMMENT?=		${ECHO_MSG} >/dev/null '***'

DISTINFO_FILE?=		${.CURDIR}/distinfo

# Debugging levels for this file, dependent on PKG_DEBUG_LEVEL definition
# 0 == normal, default, quiet operation
# 1 == all shell commands echoed before invocation
# 2 == shell "set -x" operation
PKG_DEBUG_LEVEL=	0
_PKG_SILENT=		@
_PKG_DEBUG=		# empty
_PKG_DEBUG_SCRIPT=	# empty

.if ${PKG_DEBUG_LEVEL} > 0
_PKG_SILENT=		# empty
.endif

.if ${PKG_DEBUG_LEVEL} > 1
_PKG_DEBUG=		set -x;
_PKG_DEBUG_SCRIPT=	${SH} -x
.endif

GODI_VERIFY_PLIST ?= 	no
EXTRACT_SUFX?=		.tar.gz
PREFIX=			${LOCALBASE}

# Files to create for versioning and build information
BUILD_VERSION_FILE=	${WRKDIR}/.build_version
PATCH_VERSION_FILE=	${WRKDIR}/.patch_version
BUILD_INFO_FILE=	${WRKDIR}/.build_info

# File to denote "no deletion of a package"
PRESERVE_FILE=		${WRKDIR}/.PRESERVE

# Source fingerprint: MD5 of the build.tgz file
SRC_FPRINT_FILE=	${PKGDIR}/.src_fingerprint
.if exists(${SRC_FPRINT_FILE})
SRC_FPRINT=		${SRC_FPRINT_FILE:f}
.endif

PKG_SUFX?=		.tgz

_PKGSRC_IN=		===

# How to do nothing.  Override if you, for some strange reason, would rather
# do something.
DO_NADA=		${TRUE}

AUTOGENERATE_IMAGE?=	${WRKDIR}/image


######################################################################
# ENVIRONMENTS
######################################################################

# Environment for essential programs:
ESSENTIAL_ENV=		HAVE_ESSENTIALS=yes
.for _var in ${ESSENTIAL_PROGRAMS}
_value:=		${${_var}}
ESSENTIAL_ENV:=		${ESSENTIAL_ENV} ${_var}=${_value:Q}
.endfor

.if defined(W32PORT) && ${W32PORT} == "mingw" && ( !defined(MINGW_BASE_BUILD) || ${MINGW_BASE_BUILD} == "" )

.if defined(C_INCLUDE_PATH)
C_INCLUDE_PATH:=${LOCALBASE}/include:${C_INCLUDE_PATH}
.else
C_INCLUDE_PATH=${LOCALBASE}/include
.endif
MINGW_AUTOTOOLS_ENV+=	C_INCLUDE_PATH=${C_INCLUDE_PATH}

.if defined(LIBRARY_PATH)
LIBRARY_PATH:=${LOCALBASE}/lib:${LIBRARY_PATH}
.else
LIBRARY_PATH=${LOCALBASE}/lib
.endif
MINGW_AUTOTOOLS_ENV+=	LIBRARY_PATH=${LIBRARY_PATH}

# try to enforce that the right, and currently only supported compiler is used

# AS, AR,... are honoured by  recent versions of autools based scripts, but also
# many other
# MINGW_TOOL_PREFIX should be add to godi.conf by the bootstrap script
.if defined(MINGW_TOOL_PREFIX) && ${MINGW_TOOL_PREFIX} != ""
CPPFLAGS+=			-I${LOCALBASE}/include
.if !defined(NO_MINGW_TOOLS_PREFIX)
MINGW_AUTOTOOLS_ENV+=		AR=${MINGW_TOOL_PREFIX}ar
MINGW_AUTOTOOLS_ENV+=		AS=${MINGW_TOOL_PREFIX}as
# better set in /opt/godi/etc/godi.conf
# MINGW_AUTOTOOLS_ENV+=		CC=${MINGW_TOOL_PREFIX}gcc
MINGW_AUTOTOOLS_ENV+=		CPPFILT=${MINGW_TOOL_PREFIX}c++filt
MINGW_AUTOTOOLS_ENV+=		CXX=${MINGW_TOOL_PREFIX}g++
MINGW_AUTOTOOLS_ENV+=		DLLTOOL=${MINGW_TOOL_PREFIX}dlltool
MINGW_AUTOTOOLS_ENV+=		DLLWRAP=${MINGW_TOOL_PREFIX}dllwrap
MINGW_AUTOTOOLS_ENV+=		GCOV=${MINGW_TOOL_PREFIX}gcov
MINGW_AUTOTOOLS_ENV+=		LD=${MINGW_TOOL_PREFIX}ld
MINGW_AUTOTOOLS_ENV+=		NM=${MINGW_TOOL_PREFIX}nm
MINGW_AUTOTOOLS_ENV+=		OBJCOPY=${MINGW_TOOL_PREFIX}objcopy
MINGW_AUTOTOOLS_ENV+=		OBJDUMP=${MINGW_TOOL_PREFIX}objdump
MINGW_AUTOTOOLS_ENV+=		RANLIB=${MINGW_TOOL_PREFIX}ranlib
MINGW_AUTOTOOLS_ENV+=		RC=${MINGW_TOOL_PREFIX}windres
MINGW_AUTOTOOLS_ENV+=		READELF=${MINGW_TOOL_PREFIX}readelf
MINGW_AUTOTOOLS_ENV+=		SIZE=${MINGW_TOOL_PREFIX}size
MINGW_AUTOTOOLS_ENV+=		STRINGS=${MINGW_TOOL_PREFIX}strings
MINGW_AUTOTOOLS_ENV+=		STRIP=${MINGW_TOOL_PREFIX}strip
MINGW_AUTOTOOLS_ENV+=		WINDMC=${MINGW_TOOL_PREFIX}windmc
MINGW_AUTOTOOLS_ENV+=		WINDRES=${MINGW_TOOL_PREFIX}windres
CPP=				${MINGW_TOOL_PREFIX}cpp
.else
CPP=
.endif

MAKE_ENV+=	${MINGW_AUTOTOOLS_ENV}
CONFIGURE_ENV+=	${MINGW_AUTOTOOLS_ENV}

MAKE_ENV+=	PKG_CONFIG_LIBDIR=${LOCALBASE_NATIVE:Q}/lib/pkgconfig
CONFIGURE_ENV+= PKG_CONFIG_LIBDIR=${LOCALBASE_NATIVE:Q}/lib/pkgconfig
.endif

#.if defined(MINGW_BIN_PATH) && ${MINGW_BIN_PATH} != ""
# eg. /usr/x86_64-w64-mingw32/bin contains unprefixd versions of ld , strip, ...
# however there is no unprefixed version for e.g. gcc
# MINGW_BIN_PATH should be add to godi.conf by the bootsrap script
#PATH:=		${MINGW_BIN_PATH}:${PATH}
#.endif

.if defined(MINGW_AUTOTOOLS_BUILD) && defined(MINGW_HOST) && ${MINGW_HOST} != ""
# the x86_64-w64-mingw32/it686-w64-mingw32 packages from cygwin are cross
# compilers. They are 32-bit cygwin programs, but can(!) compile to
# code that is independend of cygwin. We need to tell configure scripts
# that we want to do this.
# MINGW_HOST should be add to godi.conf by the bootstrap script.
# User can change it, if they want to generate faster code that is
# restricted to their cpu
.if defined(CYGWIN_WORDSIZE) && ${CYGWIN_WORDSIZE} == 64
CONFIGURE_ARGS+=	--host=${MINGW_HOST} --build=x86_64-pc-cygwin
.else
CONFIGURE_ARGS+=	--host=${MINGW_HOST} --build=i686-pc-cygwin
.endif

.endif

.if defined(USE_OMAKE)
MAKE_FLAGS+= 	LDFLAGS=${LDFLAGS:Q}
MAKE_FLAGS+= 	CFLAGS=${CFLAGS:Q}
.endif


.endif

.if !defined(MINGW_BASE_BUILD) || ${MINGW_BASE_BUILD} == ""

# How to invoke "make" in wrkdir to build and install the software:
MAKE_ENV+=		OPSYS=${OPSYS:Q}
.if defined(W32PORT)
MAKE_ENV+=		W32PORT=${W32PORT:Q}
.endif
MAKE_ENV+=		LDFLAGS=${LDFLAGS:Q}
MAKE_ENV+=		CREATE_SHLIBS=${CREATE_SHLIBS:Q}
MAKE_ENV+=		SHLIB_TYPE=${SHLIB_TYPE:Q}
MAKE_ENV+=		ELF_RPATH=${ELF_RPATH:Q}
.if ${ELF_RPATH} == "yes"
MAKE_ENV+=		ELF_RPATH_FLAG=${ELF_RPATH_FLAG:Q}
MAKE_ENV+=		RPATH_FLAG=${RPATH_FLAG:Q}
.endif
MAKE_ENV+=		WHOLE_ARCHIVE_FLAG=${WHOLE_ARCHIVE_FLAG:Q}
MAKE_ENV+=		NO_WHOLE_ARCHIVE_FLAG=${NO_WHOLE_ARCHIVE_FLAG:Q}
MAKE_ENV+=		LINK_ALL_LIBGCC_HACK=${LINK_ALL_LIBGCC_HACK:Q}

.if defined(W32PORT) && ${W32PORT} == "mingw"
# We always need a backup copy of flexdll because of circular dependencies
# The files at ${LOCALBASE:Q}/lib/flexdll-backup are added during the deinstallation
# of godi-flexdll
MAKE_ENV+=		PATH=${LOCALBASE:Q}/bin:${LOCALBASE:Q}/lib/flexdll-backup/bin:${PATH:Q}
.else
MAKE_ENV+=		PATH=${LOCALBASE:Q}/bin:${PATH:Q}
.endif

MAKE_ENV+=		PREFIX=${PREFIX:Q}
MAKE_ENV+= 		LOCALBASE=${LOCALBASE:Q}
MAKE_ENV+= 		LOCALBASE_NATIVE=${LOCALBASE_NATIVE:Q}
MAKE_ENV+=		CFLAGS=${CFLAGS:Q}
MAKE_ENV+=		CPPFLAGS=${CPPFLAGS:Q}
.if defined(CC)
MAKE_ENV+=		CC=${CC:Q}
.endif
.if defined(CPP)
MAKE_ENV+=		CPP=${CPP:Q}
.endif
# export the flags needed to compile and link pthreaded code.
# GODI: Probably not needed.
MAKE_ENV+=		PTHREAD_CFLAGS=${PTHREAD_CFLAGS:Q}
MAKE_ENV+=		PTHREAD_LDFLAGS=${PTHREAD_LDFLAGS:Q}
# Add essentials if requested:
.if defined(MAKE_GETS_ESSENTIALS) && ${MAKE_GETS_ESSENTIALS} == "yes"
MAKE_ENV+=		${ESSENTIAL_ENV}
.endif

.endif # MINGW_BASE_BUILD

# Extra variables for build:
BUILD_ENV?=		${MAKE_ENV}

# Extra variables for test:
TEST_ENV?=		${MAKE_ENV}

# Extra variables for install:
INSTALL_ENV?=		${MAKE_ENV}

.if !defined(MINGW_BASE_BUILD) || ${MINGW_BASE_BUILD} == ""
# How to invoke "configure" in wrkdir to configure the software:
CONFIGURE_ENV+=		OPSYS=${OPSYS:Q}
.if defined(W32PORT)
CONFIGURE_ENV+=		W32PORT=${W32PORT:Q}
.endif
CONFIGURE_ENV+=		LDFLAGS=${LDFLAGS:Q}
CONFIGURE_ENV+=		CREATE_SHLIBS=${CREATE_SHLIBS:Q}
CONFIGURE_ENV+=		SHLIB_TYPE=${SHLIB_TYPE:Q}
CONFIGURE_ENV+=		ELF_RPATH=${ELF_RPATH:Q}
.if ${ELF_RPATH} == "yes"
CONFIGURE_ENV+=		ELF_RPATH_FLAG=${ELF_RPATH_FLAG:Q}
CONFIGURE_ENV+=		RPATH_FLAG=${RPATH_FLAG:Q}
.endif
.if defined(W32PORT) && ${W32PORT} == "mingw"
# see corresponding comment at MAKE_ENV
CONFIGURE_ENV+=		PATH=${LOCALBASE:Q}/bin:${LOCALBASE:Q}/lib/flexdll-backup/bin:${PATH:Q}
.else
CONFIGURE_ENV+=		PATH=${LOCALBASE:Q}/bin:${PATH:Q}
.endif
CONFIGURE_ENV+=		PREFIX=${PREFIX:Q}
CONFIGURE_ENV+= 	LOCALBASE=${LOCALBASE:Q}
CONFIGURE_ENV+= 	LOCALBASE_NATIVE=${LOCALBASE_NATIVE:Q}
CONFIGURE_ENV+=		CFLAGS=${CFLAGS:Q}
CONFIGURE_ENV+=		CPPFLAGS=${CPPFLAGS:Q}
CONFIGURE_ENV+=		INSTALL=${INSTALL:Q}\ -c
CONFIGURE_ENV+=		ac_given_INSTALL=${INSTALL:Q}\ -c
CONFIGURE_ENV+=		INSTALL_DATA=${INSTALL_DATA:Q}
CONFIGURE_ENV+=		INSTALL_PROGRAM=${INSTALL_PROGRAM:Q}
# breaks autotools
#CONFIGURE_ENV+=	INSTALL_SCRIPT=${INSTALL_SCRIPT:Q}
CONFIGURE_ENV+=		M4=${M4:Q}
CONFIGURE_ENV+=		AWK=${AWK:Q}
.if defined(CC)
CONFIGURE_ENV+=		CC=${CC:Q}
.endif
.if defined(CPP)
CONFIGURE_ENV+=		CPP=${CPP:Q}
.endif

.endif # MINGW_BASE_BUILD

# Passed to most of script invocations
SCRIPTS_ENV+= CURDIR=${.CURDIR:Q} DISTDIR=${DISTDIR:Q} \
	PATH=${LOCALBASE:Q}/bin:${PATH:Q} \
	WRKDIR=${WRKDIR:Q} WRKSRC=${WRKSRC:Q} PATCHDIR=${PATCHDIR:Q} \
	SCRIPTDIR=${SCRIPTDIR:Q} FILESDIR=${FILESDIR:Q} \
	_PKGSRCDIR=${_PKGSRCDIR:Q} DEPENDS=${DEPENDS:Q} \
	PREFIX=${PREFIX:Q} LOCALBASE=${LOCALBASE:Q}

# Add these defs to the ones dumped into +BUILD_DEFS
BUILD_DEFS+=	PKGPATH
BUILD_DEFS+=	OPSYS W32PORT OS_VERSION MACHINE_ARCH MACHINE_GNU_ARCH
BUILD_DEFS+=	CPPFLAGS CFLAGS LDFLAGS
BUILD_DEFS+=	CONFIGURE_ENV CONFIGURE_ARGS

LOCALBASE_DIRS = bin doc etc gui lib libexec man sbin share var include

######################################################################
# CONFIGURE STAGE
######################################################################


# --- GNU configure ---

.if defined(GNU_CONFIGURE)
#
# CONFIGURE_PREFIX is the argument to the --prefix option passed to the
# GNU configure script.
#
GNU_CONFIGURE_PREFIX?=	${PREFIX}
.if defined(SET_GNU_HOST) && ${SET_GNU_HOST} == "yes"
CONFIGURE_ARGS+=	--host=${MACHINE_GNU_PLATFORM:Q}
.endif
CONFIGURE_ARGS+=	--prefix=${GNU_CONFIGURE_PREFIX:Q}
HAS_CONFIGURE=		yes
# If GNU_CONFIGURE is defined, then pass LIBS to the GNU configure script.
# also pass in a CONFIG_SHELL to avoid picking up bash
#CONFIGURE_ENV+=	LIBS="${LIBS}"
#CONFIG_SHELL?=		${SH}
#CONFIGURE_ENV+=	CONFIG_SHELL=${CONFIG_SHELL}
.endif


# --- GODI configure scripts (godi-script) ---

# GODI's flavor of configure scripts:
.if defined(CONF_SCRIPT) || defined(CONF_SCRIPT_TEXT)
HAS_CONFIGURE=yes
CONFIGURE_GETS_ESSENTIALS=yes
CONFIGURE_ENV+=	SEARCH_LIBS=${SEARCH_LIBS:Q}
CONFIGURE_ENV+= PKGBASE=${PKGBASE:Q}
GODI_SCRIPT ?= ${LOCALBASE}/bin/godi_script

.if defined(CONF_SCRIPT) && ${CONF_SCRIPT:M*.gcs} != ""

.if defined(W32PORT) && ${W32PORT} == "mingw"
CONF_SCRIPT_REAL=${CYGWIN_ROOT}${.CURDIR}/${CONF_SCRIPT}
.else
CONF_SCRIPT_REAL=${.CURDIR}/${CONF_SCRIPT}
.endif

CONFIGURE_SCRIPT:=${GODI_SCRIPT} ${.CURDIR}/${CONF_SCRIPT_REAL}
BUILD_DEPENDS += godi-core-script-*
.else
.  if defined(CONF_SCRIPT_TEXT)
CONF_SCRIPT_FULL_TEXT=
.    text
.    expand
open Godi_script
open Godi_conf
module M = Make(struct
  $(CONF_SCRIPT_TEXT)
end)
.    endtext
CONFIGURE_SCRIPT:=${GODI_SCRIPT} -c ${CONF_SCRIPT_FULL_TEXT:Q}
BUILD_DEPENDS += godi-core-script>=3.0.18
.  else
.if defined(W32PORT) && ${W32PORT} == "mingw"
CONFIGURE_SCRIPT=${CYGWIN_ROOT}${.CURDIR}/${CONF_SCRIPT}
.else
CONFIGURE_SCRIPT=${.CURDIR}/${CONF_SCRIPT}
.endif
.  endif
.endif

CONFIGURE_ARGS:=${CONF_SCRIPT_ARGS}
.endif

### WINGMAKE ###
.if defined(WINGMAKE) && ${WINGMAKE} == "yes"
MAINTAINER?=		gmake@ml.ignorelist.com
MAKE_PROGRAM=		${GMAKE}

IMAGE_DIR=		${WRKDIR}/image

AUTOGENERATE_PLIST=	yes
AUTOGENERATE_IMAGE=	${CYGWIN_ROOT}${IMAGE_DIR}${LOCALBASE}

CONFIGURE_ENV+=		${BUILD_OCAMLFIND_ENV}
MAKE_ENV+=		${BUILD_OCAMLFIND_ENV}

CONFIGURE_ENV+=		OCAMLFIND_DESTDIR=${AUTOGENERATE_IMAGE:Q}/lib/ocaml/pkg-lib
MAKE_ENV+=		OCAMLFIND_DESTDIR=${AUTOGENERATE_IMAGE:Q}/lib/ocaml/pkg-lib
INSTALL_ENV+=		OCAMLFIND_DESTDIR=${AUTOGENERATE_IMAGE:Q}/lib/ocaml/pkg-lib

CONFIGURE_SCRIPT?=	${MAKE} -C ${PKGDIR:Q} ${MAKEFLAGS} wingmake-configure

.if defined(HAS_CONFIGURE) && ${HAS_CONFIGURE} == "yes"
WINGMAKE_CONFIGURE?=	${WRKSRC}/configure
.else
WINGMAKE_CONFIGURE?=	no
HAS_CONFIGURE=		yes
.endif

wingmake-configure:
.script
.  import LOCALBASE WRKSRC PKGDIR CHMOD
.  import WINGMAKE_CONFIGURE CONFIGURE_ARGS PAX
.  expand
	${_PKG_SILENT}${_PKG_DEBUG}
.  noexpand
set -e
unset DEBUG PROFILE TEST TESTS
if [ -d "${PKGDIR}/files" ]; then
  files_found=0
  cd "${PKGDIR}/files"
  for f in * .* ; do
    if [ -e "$f" ]; then
      files_found=1;
      break;
    fi
  done
  if [ $files_found -eq 1 ]; then
    ${PAX} -rw -pp . "${WRKSRC}"
  fi
fi
cd "$WRKSRC"
if [ "$WINGMAKE_CONFIGURE" != "no" ]; then
  case "${WINGMAKE_CONFIGURE}" in
    "${WRKSRC}"*)
	if [ -f "${WINGMAKE_CONFIGURE}" ]; then
          ${CHMOD} 0755 "${WINGMAKE_CONFIGURE}"
	fi
	;;
  esac

  if [ -z "$CONFIGURE_ARGS" ]; then
	${WINGMAKE_CONFIGURE}
  else
	${WINGMAKE_CONFIGURE} ${CONFIGURE_ARGS}
  fi
fi
.endscript

MAKE_FLAGS+=	DESTDIR=${IMAGE_DIR:Q}
INSTALL_SCRIPT?= ${GMAKE} -C ${WRKSRC:Q} ${MAKE_FLAGS} -f ${MAKEFILE:Q} ${INSTALL_TARGET} && \
	${MAKE} -C ${PKGDIR:Q} ${MAKEFLAGS} destdir-copy-docs

.endif
### END WINGMAKE ###

### START WINPKG ###
.if defined(WINPKG) && ${WINPKG} == "yes"
BUILD_DEPENDS+= godi-opam-installer>=1.0:../../godi/godi-opam-installer
BUILD_DEPENDS+= godi-findlib>=1.0:../../godi/godi-findlib

MAINTAINER?=		pkg@ml.ignorelist.com
IMAGE_DIR=		${WRKDIR}/image

AUTOGENERATE_PLIST=	yes
AUTOGENERATE_IMAGE=	${CYGWIN_ROOT}${IMAGE_DIR}${LOCALBASE}

CONFIGURE_ENV+=		${BUILD_OCAMLFIND_ENV}
MAKE_ENV+=		${BUILD_OCAMLFIND_ENV}

CONFIGURE_ENV+=		OCAMLFIND_DESTDIR=${AUTOGENERATE_IMAGE:Q}/lib/ocaml/pkg-lib
MAKE_ENV+=		OCAMLFIND_DESTDIR=${AUTOGENERATE_IMAGE:Q}/lib/ocaml/pkg-lib
INSTALL_ENV+=		OCAMLFIND_DESTDIR=${AUTOGENERATE_IMAGE:Q}/lib/ocaml/pkg-lib

CONFIGURE_SCRIPT?=	${MAKE} -C ${PKGDIR:Q} ${MAKEFLAGS} winpkg-configure

.if defined(HAS_CONFIGURE) && ${HAS_CONFIGURE} == "yes"
WINPKG_CONFIGURE?=	${WRKSRC}/configure
.else
WINPKG_CONFIGURE?=	no
HAS_CONFIGURE=		yes
.endif

winpkg-configure:
.script
.  import LOCALBASE WRKSRC PKGDIR CHMOD
.  import WINPKG_CONFIGURE CONFIGURE_ARGS PAX
.  expand
	${_PKG_SILENT}${_PKG_DEBUG}
.  noexpand
set -e
unset DEBUG PROFILE TEST TESTS
if [ -d "${PKGDIR}/files" ]; then
  files_found=0
  cd "${PKGDIR}/files"
  for f in * .* ; do
    if [ -e "$f" ]; then
      files_found=1;
      break;
    fi
  done
  if [ $files_found -eq 1 ]; then
    ${PAX} -rw -pp . "${WRKSRC}"
  fi
fi
cd "$WRKSRC"
if [ "$WINPKG_CONFIGURE" != "no" ]; then
  case "${WINPKG_CONFIGURE}" in
    "${WRKSRC}"*)
	if [ -f "${WINPKG_CONFIGURE}" ]; then
          ${CHMOD} 0755 "${WINPKG_CONFIGURE}"
	fi
	;;
  esac

  if [ -z "$CONFIGURE_ARGS" ]; then
	${WINPKG_CONFIGURE}
  else
	${WINPKG_CONFIGURE} ${CONFIGURE_ARGS}
  fi
fi
.endscript


WINPKG_BUILD_SCRIPT?=	${CYGWIN_ROOT:Q}${WRKSRC:Q}/pkg/build.ml

BUILD_SCRIPT=	${LOCALBASE:Q}/bin/ocaml.exe ${WINPKG_BUILD_SCRIPT}

.if !defined(${GODI_WINPKG_NO_OPT}) && ${GODI_HAVE_OCAMLOPT} == "yes"
BUILD_SCRIPT+=  native=true
.if !defined(${GODI_WINPKG_NO_NATIVE_DYNLINK})
BUILD_SCRIPT+=  native-dynlink=true
.endif
.endif


WINPKG_INSTALL_FILE?=	${CYGWIN_ROOT:Q}${WRKSRC:Q}/${PKGBASE:C/^[^-]*-//}.install

INSTALL_SCRIPT?= ${MAKE} -C ${PKGDIR:Q} ${MAKEFLAGS} destdir-copy-docs && \
	${SETENV} ${INSTALL_ENV} ${MAKE} -C ${PKGDIR:Q} ${MAKEFLAGS} winpkg-install

winpkg-install:
.script
.  import FIND LOCALBASE RM WRKSRC MV
.  import WINPKG_INSTALL_FILE
.  import AUTOGENERATE_IMAGE
.  import MKDIR FILE_CMD BASENAME
.  import PKGBASE
.  expand
	${_PKG_SILENT}${_PKG_DEBUG}
.  noexpand
set -e

cd "${WRKSRC}"
"${LOCALBASE}/sbin/opam-installer.exe" --prefix="${AUTOGENERATE_IMAGE}" "${WINPKG_INSTALL_FILE}"
cd "$(cygpath "${AUTOGENERATE_IMAGE}")"
${FIND} . -type d -empty -delete
${MKDIR} -p lib/ocaml/pkg-lib
${MKDIR} -p doc

if [ -d "bin" ]; then
    cd bin
    for f in * ; do
        [ ! -f "$f" ] && continue
        case "$f" in
            *.*)
                continue ;;
            *)
		type="$(${FILE_CMD} -b "$f")"
		case "$type" in
		  *text*) continue;;
		  *script*) continue ;;
		  *executable*)
                    mv "$f" "${f}.exe" ;;
                esac
        esac
    done
    cd ..
fi
cd lib
for f in * ; do
    [ ! -d "$f" ] && continue
    if [ "$f" != "ocaml" ]; then
       mv "$f" ocaml/pkg-lib
    fi
done
cd ..
${FIND} . -type d -empty -delete
if [ -d doc ]; then
  cd doc
  for d in * ; do
    [ ! -d "$d" ] && continue
    cd "$d"
    if [ "$d" = "$PKGBASE" ]; then
      basename="$($BASENAME "${WINPKG_INSTALL_FILE}")"
      if [ -n "$basename" ] && [ -f "$basename" ]; then
        ${RM} -f "$basename"
      fi
    else
      ${MV} -f ./* "../${PKGBASE}"
    fi
    cd ..
  done
  cd ..
fi
${FIND} . -type d -empty -delete
.endscript


.endif
### END WINPKG ###




# --- OASIS ---

.if defined(OASIS) && ${OASIS} == "yes"
HAS_CONFIGURE=		yes
OASIS_SETUP?=		yes

.if ${OASIS_SETUP} == "yes"
BUILD_DEPENDS+=		apps-oasis>=0
CONFIGURE_SCRIPT?=	${LOCALBASE}/bin/oasis setup && \
                        ${LOCALBASE}/bin/ocaml setup.ml -configure
.else
CONFIGURE_SCRIPT?=	${LOCALBASE}/bin/ocaml setup.ml -configure
.endif
BUILD_SCRIPT?=		${LOCALBASE}/bin/ocaml setup.ml -build && \
                        { ${ECHO} "*** Building documentation"; \
			  ${LOCALBASE}/bin/ocaml setup.ml -doc || \
			  ${ECHO} "*** Ignoring failed doc build"; }
INSTALL_SCRIPT?=	${LOCALBASE}/bin/ocaml setup.ml -install

CONFIGURE_ARGS+=	--prefix ${LOCALBASE:Q}
CONFIGURE_ARGS+=	--mandir ${LOCALBASE:Q}/man
CONFIGURE_ARGS+=	--docdir ${LOCALBASE:Q}/doc/${PKGBASE:Q}
CONFIGURE_ARGS+=	--destdir ${WRKDIR:Q}/image

AUTOGENERATE_PLIST=	yes
AUTOGENERATE_IMAGE=	${WRKDIR}/image/${LOCALBASE}
.endif

# --- WINOASIS ---
.if defined(WINOASIS) && ${WINOASIS} == "yes"
HAS_CONFIGURE=		yes
WINOASIS_SETUP?=	no
WINOASIS_DISABLE_DEBUG?=no
WINOASIS_DISABLE_DOCS?=	no
WINOASIS_DISABLE_TESTS?=no
WINOASIS_CMT?=		yes
WINOASIS_USE_BASH?=	yes
MAINTAINER?=		oasis@ml.ignorelist.com

CONFIGURE_ENV+=	${BUILD_OCAMLFIND_ENV}
MAKE_ENV+=	${BUILD_OCAMLFIND_ENV}

IMAGE_DIR=		${WRKDIR}/image
CONFIGURE_ARGS+= --prefix ${LOCALBASE:Q}
CONFIGURE_ARGS+= --docdir ${LOCALBASE:Q}/doc/${PKGBASE:Q}
CONFIGURE_ARGS+= --mandir  ${LOCALBASE:Q}/man
CONFIGURE_ARGS+= --datadir ${LOCALBASE:Q}/share
CONFIGURE_ARGS+= --destdir ${CYGWIN_ROOT:Q}${WRKDIR:Q}/image

AUTOGENERATE_PLIST=	yes
AUTOGENERATE_IMAGE=	${CYGWIN_ROOT}${IMAGE_DIR}${LOCALBASE}
INSTALL_ENV+=		OCAMLFIND_DESTDIR=${AUTOGENERATE_IMAGE:Q}/lib/ocaml/pkg-lib

.if ${WINOASIS_DISABLE_TESTS} == "yes"
CONFIGURE_ARGS+= --disable-tests
.elif ${WINOASIS_DISABLE_TESTS} == "no"
CONFIGURE_ARGS+= --enable-tests
.endif

.if ${WINOASIS_DISABLE_DOCS} == "yes"
CONFIGURE_ARGS+= --disable-docs
.elif ${WINOASIS_DISABLE_DOCS} == "no"
CONFIGURE_ARGS+= --enable-docs
.endif

.if ${WINOASIS_DISABLE_DEBUG} == "yes"
CONFIGURE_ARGS+= --disable-debug
.elif ${WINOASIS_DISABLE_DEBUG} == "no"
CONFIGURE_ARGS+= --enable-debug
.endif

.if ${WINOASIS_USE_BASH} == "yes"
CONFIGURE_ARGS+= --use-bash ${CYGWIN_ROOT:Q}/bin/dash.exe
.endif

.if ${WINOASIS_SETUP} == "yes"
BUILD_DEPENDS+= apps-oasis>0.3.0:../../apps/apps-oasis
.endif

CONFIGURE_SCRIPT?= ${MAKE} -C ${PKGDIR:Q} ${MAKEFLAGS} winoasis-configure

winoasis-configure:
.script
.  import LOCALBASE WRKSRC PKGDIR
.  import WINOASIS_SETUP CONFIGURE_ARGS WINOASIS_CMT
.  expand
	${_PKG_SILENT}${_PKG_DEBUG}
.  noexpand
set -e
unset DEBUG PROFILE TEST TESTS
cd "$WRKSRC"
if [ "$WINOASIS_SETUP" = "yes" ]; then
	${LOCALBASE}/bin/oasis setup
elif [ -f "${PKGDIR}/files/oasis_setup.ml" ]; then
	cp "${PKGDIR}/files/oasis_setup.ml" "${WRKSRC}/setup.ml"
	if [ -f "${PKGDIR}/files/oasis_myocamlbuild.ml" ] && [ -f "${WRKSRC}/myocamlbuild.ml" ]; then
		cp "${PKGDIR}/files/oasis_myocamlbuild.ml" "${WRKSRC}/myocamlbuild.ml"
	fi
fi
if [ "$WINOASIS_CMT" = "yes" ]; then
  if [ -z "$OCAMLPARAM" ]; then
     OCAMLPARAM=bin-annot=1,_
  else
     case "${OCAMLPARAM}" in
      _*)
	OCAMLPARAM="${OCAMLPARAM},bin-annot=1" ;;
      *)
	OCAMLPARAM="bin-annot=1,${OCAMLPARAM}" ;;
     esac
  fi
  export OCAMLPARAM
fi

${LOCALBASE}/bin/ocaml -w -A setup.ml -configure $CONFIGURE_ARGS
.endscript



BUILD_SCRIPT?=	${MAKE} -C ${PKGDIR:Q} ${MAKEFLAGS} winoasis-build

winoasis-build:
.script
.  import LOCALBASE WRKSRC WINOASIS_CMT
.  import WINOASIS_DISABLE_TESTS WINOASIS_DISABLE_DOCS
.  expand
	${_PKG_SILENT}${_PKG_DEBUG}
.  noexpand
set -e
unset DEBUG PROFILE TEST TESTS
if [ "$WINOASIS_CMT" = "yes" ]; then
  if [ -z "$OCAMLPARAM" ]; then
     OCAMLPARAM=bin-annot=1,_
  else
     case "${OCAMLPARAM}" in
      _*)
	OCAMLPARAM="${OCAMLPARAM},bin-annot=1" ;;
      *)
	OCAMLPARAM="bin-annot=1,${OCAMLPARAM}" ;;
     esac
  fi
  export OCAMLPARAM
fi
cd "$WRKSRC"
"${LOCALBASE}/bin/ocaml" -w -A setup.ml -build
if [ "$WINOASIS_DISABLE_TESTS" = "no"  ]; then
  "${LOCALBASE}/bin/ocaml" -w -A setup.ml -test
fi
if [ "$WINOASIS_DISABLE_DOCS" = "no" ]; then
  "${LOCALBASE}/bin/ocaml" -w -A setup.ml -doc
fi
.endscript

#INSTALL_SCRIPT?= ${MAKE} -C ${PKGDIR:Q} ${MAKEFLAGS} winoasis-install
#winoasis-install:
#	@cd ${WRKSRC:Q} && ${SETENV} ${INSTALL_ENV} ${LOCALBASE}/bin/ocaml -w -A setup.ml -install
#	@${MAKE} -C ${PKGDIR:Q} ${MAKEFLAGS} destdir-copy-docs

INSTALL_SCRIPT?= ${TRUE} && cd ${WRKSRC:Q} && \
	${SETENV} ${INSTALL_ENV} ${LOCALBASE}/bin/ocaml -w -A setup.ml -install && \
	${MAKE} -C ${PKGDIR:Q} ${MAKEFLAGS} destdir-copy-docs

.endif
# --- END WINOASIS ---


# --- other ---

# Add essentials if requested:
.if defined(CONFIGURE_GETS_ESSENTIALS) && ${CONFIGURE_GETS_ESSENTIALS} == "yes"
CONFIGURE_ENV+=		${ESSENTIAL_ENV}
.endif


# temporary hack until default manifest is shipped by mingw-w64 itself
.if !defined(NO_FLEXLINK_MANIFEST_HACK)
CONFIGURE_ENV+=		FLEXLINK_MANIFEST_HACK=yes
MAKE_ENV+=		FLEXLINK_MANIFEST_HACK=yes
INSTALL_ENV+=		FLEXLINK_MANIFEST_HACK=yes
.endif

# Config file related settings - see Packages.txt
#
PKG_SYSCONFVAR?=	${PKGBASE}
PKG_SYSCONFBASE?=	${PREFIX}/etc
#.if defined(PKG_SYSCONFDIR.${PKG_SYSCONFVAR})
#PKG_SYSCONFDIR=		${PKG_SYSCONFDIR.${PKG_SYSCONFVAR}}
#.else
PKG_SYSCONFSUBDIR?=	# empty
.  if empty(PKG_SYSCONFSUBDIR)
PKG_SYSCONFDIR=		${PKG_SYSCONFBASE}
.  else
PKG_SYSCONFDIR=		${PKG_SYSCONFBASE}/${PKG_SYSCONFSUBDIR}
.  endif
#.endif

CONFIGURE_ENV+=		PKG_SYSCONFDIR=${PKG_SYSCONFDIR:Q}
MAKE_ENV+=		PKG_SYSCONFDIR=${PKG_SYSCONFDIR:Q}
BUILD_DEFS+=		PKG_SYSCONFDIR

######################################################################
# SANDBOX
######################################################################

# This is for the Gentoo sandbox. The purpose is just to prevent
# packaging errors. This does _not_ add any security!

ENABLE_SANDBOX ?=	no

.if ${ENABLE_SANDBOX} == "yes"
SANDBOX_WRAPPER:= 	"${.PARSEDIR}/sandbox_wrapper.sh"
SANDBOX_CMD ?=		${SANDBOX_WRAPPER}
SANDBOX_CONFIGURE_CMD=  ${SANDBOX_CMD}
SANDBOX_BUILD_CMD=	${SANDBOX_CMD}
SANDBOX_TEST_CMD=	${SANDBOX_CMD}
SANDBOX_INSTALL_CMD=	${SANDBOX_CMD}

SANDBOX_WRITE:=		${SANDBOX_WRITE}:/tmp:/var/tmp

CONFIGURE_ENV+=		SANDBOX_READ=/
CONFIGURE_ENV+=		SANDBOX_WRITE=${SANDBOX_WRITE}:${WRKDIR:Q}

BUILD_ENV+=		SANDBOX_READ=/
BUILD_ENV+=		SANDBOX_WRITE=${SANDBOX_WRITE}:${WRKDIR:Q}
TEST_ENV+=		SANDBOX_READ=/
TEST_ENV+=		SANDBOX_WRITE=${SANDBOX_WRITE}:${WRKDIR:Q}

INSTALL_ENV+=		SANDBOX_READ=/

__tmp__=		${WRKDIR:Q}
.ifndef AUTOGENERATE_PLIST
.for d in ${LOCALBASE_DIRS}
__tmp__:=		${__tmp__}:${LOCALBASE:Q}/${d:Q}
.endfor
.endif

INSTALL_ENV+=		SANDBOX_WRITE=${SANDBOX_WRITE}:${__tmp__}

.else
SANDBOX_CONFIGURE_CMD=
SANDBOX_BUILD_CMD=
SANDBOX_TEST_CMD=
SANDBOX_INSTALL_CMD=
.endif

######################################################################
# PATCH
######################################################################

# Setup for ${PATCH}:
# _PATCH_RMEMPTY, _PATCH_IGNREV, _PATCH_QUIET come from defs.cmd.mk
PATCH_STRIP=		-p0
PATCH_DIST_STRIP=	-p0
.if defined(PATCH_DEBUG) || defined(PKG_VERBOSE)
PATCH_DEBUG_TMP=	yes
PATCH_ARGS=		-d ${WRKSRC:Q} ${_PATCH_RMEMPTY} ${PATCH_STRIP}
PATCH_DIST_ARGS=	-d ${WRKSRC:Q} ${_PATCH_RMEMPTY} ${PATCH_DIST_STRIP}
.else
PATCH_DEBUG_TMP=	no
PATCH_ARGS=		-d ${WRKSRC:Q} ${_PATCH_IGNREV} ${_PATCH_QUIET} ${PATCH_RMEMPTY} ${PATCH_STRIP}
PATCH_DIST_ARGS=	-d ${WRKSRC:Q} ${_PATCH_IGNREV} ${_PATCH_QUIET} ${PATCH_RMEMPTY} ${PATCH_DIST_STRIP}
.endif
.if defined(_PATCH_CAN_BACKUP) && ${_PATCH_CAN_BACKUP} == "yes"
PATCH_ARGS+=		${_PATCH_BACKUP_ARG} .orig
PATCH_DIST_ARGS+=	${_PATCH_BACKUP_ARG} .orig_dist
.endif
PATCH_ARGS+=		${_PATCH_BINARY}
PATCH_FUZZ_FACTOR?=	-F0			# Default to zero fuzz

######################################################################
# INSTALL STAGE CONFIG
######################################################################

# A few aliases for *-install targets
INSTALL_PROGRAM= \
	${INSTALL} ${COPY} -m ${BINMODE}
INSTALL_DATA= \
	${INSTALL} ${COPY} -m ${SHAREMODE}
INSTALL_MAN= \
	${INSTALL} ${COPY} -m ${MANMODE}
INSTALL_PROGRAM_DIR= \
	${INSTALL} -d -m ${PKGDIRMODE}
INSTALL_DATA_DIR= \
	${INSTALL} -d -m ${PKGDIRMODE}
INSTALL_MAN_DIR= \
	${INSTALL} -d -m ${PKGDIRMODE}

INSTALL_MACROS=	BSD_INSTALL_PROGRAM=${INSTALL_PROGRAM:Q}		\
		BSD_INSTALL_DATA=${INSTALL_DATA:Q}			\
		BSD_INSTALL_MAN=${INSTALL_MAN:Q}			\
		BSD_INSTALL=${INSTALL:Q}				\
		BSD_INSTALL_PROGRAM_DIR=${INSTALL_PROGRAM_DIR:Q}	\
		BSD_INSTALL_DATA_DIR=${INSTALL_DATA_DIR:Q}		\
		BSD_INSTALL_MAN_DIR=${INSTALL_MAN_DIR:Q}
MAKE_ENV+=	${INSTALL_MACROS}
SCRIPTS_ENV+=	${INSTALL_MACROS}

######################################################################
# SUBSTITUTION CONFIG
######################################################################

# Set PLIST_SUBST to substitute "${variable}" to "value" in PLIST
# Known BUG: Quoting does not work within PLIST_SUBST due to the way
# this variable is processed.
PLIST_SUBST+=	OPSYS=${OPSYS}						\
		OS_VERSION=${OS_VERSION}				\
		MACHINE_ARCH=${MACHINE_ARCH}				\
		MACHINE_GNU_ARCH=${MACHINE_GNU_ARCH}			\
		MACHINE_GNU_PLATFORM=${MACHINE_GNU_PLATFORM}		\
		LN=${LN}						\
		LOWER_VENDOR=${LOWER_VENDOR}				\
		LOWER_OPSYS=${LOWER_OPSYS}				\
		LOWER_OS_VERSION=${LOWER_OS_VERSION}			\
		PKGBASE=${PKGBASE}					\
		PKGNAME=${PKGNAME_NOREV}				\
		PKGLOCALEDIR=${PKGLOCALEDIR}				\
		PKGVERSION=${PKGVERSION:C/nb[0-9]*$//}			\
		LOCALBASE=${LOCALBASE}					\
		SVR4_PKGNAME=${SVR4_PKGNAME}				\
		CHMOD=${CHMOD}						\
		MKDIR=${MKDIR_EXE}					\
		RMDIR=${RMDIR}						\
		RM=${RM}						\
		TRUE=${TRUE}


.if defined(MESSAGE_SRC)
MESSAGE=		${WRKDIR}/.MESSAGE

# Set MESSAGE_SUBST to substitute "${variable}" to "value" in MESSAGE
# Known BUG: Same quoting issues as for PLIST_SUBST.
MESSAGE_SUBST+=	PKGNAME=${PKGNAME}					\
		PREFIX=${PREFIX}					\
		LOCALBASE=${LOCALBASE}					\
		PKG_SYSCONFDIR=${PKG_SYSCONFDIR}
.endif


######################################################################
# TOOL SETUP
######################################################################

# Args for ptool:
PTOOL_ARGS=		${PTOOL_ARGS_NOBUILD} \
			-set-build-version-file ${BUILD_VERSION_FILE:Q} \
			-set-build-info-file ${BUILD_INFO_FILE:Q}
PTOOL_ARGS_NOBUILD=	-set-name ${PKGNAME:Q} \
			-set-comment ${COMMENT:Q} \
			-set-description-file ${DESCR:Q} \
			-set-maintainer ${MAINTAINER:Q} \
			${DEPENDS:D-set-dependencies ${DEPENDS:QQ}} \
			${BUILD_DEPENDS:D-set-build-dependencies ${BUILD_DEPENDS:QQ}} \
			${CONFLICTS:D-set-conflicts ${CONFLICTS:QQ}} \
			${INSTALL_MESSAGE_FILE:D-set-install-message-file ${INSTALL_MESSAGE_FILE:QQ}} \
			${PKG_PRESERVE:D-set-preserve true} \
			${INSTALL_FILE:D-set-install-script ${INSTALL_FILE:QQ}} \
			${DEINSTALL_FILE:D-set-deinstall-script ${DEINSTALL_FILE:QQ}} \
			${INSTALL_SCRIPT_FILE:D-set-install-script-file ${INSTALL_SCRIPT_FILE:QQ}} \
			${DEINSTALL_SCRIPT_FILE:D-set-deinstall-script-file ${DEINSTALL_SCRIPT_FILE:QQ}} \
			${SRC_FPRINT:D-set-src-fingerprint ${SRC_FPRINT:QQ}}

######################################################################
# PACKAGE CONFIG PART 2
######################################################################

ALL_TARGET?=		all
INSTALL_TARGET?=	install

CONFIGURE_SCRIPT?=	./configure
.if !defined(USE_OMAKE)
BUILD_SCRIPT?=		${MAKE_PROGRAM} ${MAKE_FLAGS} -f ${MAKEFILE:Q} \
			  ${ALL_TARGET}
INSTALL_SCRIPT?=	${MAKE_PROGRAM} ${MAKE_FLAGS} -f ${MAKEFILE:Q} \
			  ${INSTALL_TARGET}
.if defined(TEST_TARGET)
TEST_SCRIPT?=		${MAKE_PROGRAM} ${MAKE_FLAGS} -f ${MAKEFILE:Q} \
			  ${TEST_TARGET}
.endif
.else
BUILD_SCRIPT?=		${MAKE_PROGRAM} ${MAKE_FLAGS} ${ALL_TARGET}
INSTALL_SCRIPT?=	${MAKE_PROGRAM} ${MAKE_FLAGS} ${INSTALL_TARGET}
.if defined(TEST_TARGET)
TEST_SCRIPT?=		${MAKE_PROGRAM} ${MAKE_FLAGS} ${TEST_TARGET}
.endif
.endif


ALLFILES?=	${DISTFILES}
CKSUMFILES?=	${ALLFILES}

# delete $IGNOREFILES from $CKSUMFILES so that the checksum is not verified
.for __tmp__ in ${IGNOREFILES}
CKSUMFILES:=	${CKSUMFILES:N${__tmp__}}
.endfor

# accept singular and plural :-(
MASTER_SITES_OVERRIDE?= ${MASTER_SITE_OVERRIDE}
MASTER_SITES_BACKUP?= 	${MASTER_SITE_BACKUP}
MASTER_SITES_PROFILE?= 	${MASTER_SITE_PROFILE}
MASTER_SITES?=		${MASTER_SITE}

ALLSITES=	${MASTER_SITES_OVERRIDE} \
                ${MASTER_SITES_PROFILE}  \
                ${MASTER_SITES_BACKUP}   \
                ${MASTER_SITES}

# This is what is actually going to be extracted, and is overridable
#  by user.
EXTRACT_ONLY?=	${DISTFILES:S/^/${DISTPREFIX}/}

PKGREPOSITORYSUBDIR?=	All
PKGREPOSITORY=		${PACKAGES}/${PKGREPOSITORYSUBDIR}
PKGFILE=		${PKGREPOSITORY}/${PKGNAME}${PKG_SUFX}

AUTOGENERATE_IMAGE?=	${WRKSRC}/image
AUTOGENERATE_IMAGE_DIRS+= ${LOCALBASE_DIRS}
AUTOGENERATE_IMAGE_DIRS+= lib/ocaml/pkg-lib/stublibs
AUTOGENERATE_IMAGE_DIRS+= man/man1
AUTOGENERATE_IMAGE_DIRS+= man/man2
AUTOGENERATE_IMAGE_DIRS+= man/man3
AUTOGENERATE_IMAGE_DIRS+= man/man4
AUTOGENERATE_IMAGE_DIRS+= man/man5
AUTOGENERATE_IMAGE_DIRS+= man/man6
AUTOGENERATE_IMAGE_DIRS+= man/man7
AUTOGENERATE_IMAGE_DIRS+= man/man8
AUTOGENERATE_IMAGE_DIRS+= man/man9

######################################################################
# RULES START HERE
######################################################################

# Skip rules if we are only scanning the packages
######################################################################
.ifndef PACKAGE_SCANNING
######################################################################

.MAIN: all

#
# Now print some error messages that we know we should ignore the pkg
#
.  if defined(PKG_FAIL_REASON)
fetch checksum extract patch configure all build install package:
	@for str in ${PKG_FAIL_REASON} ; \
	do \
	     ${ECHO} "${_PKGSRC_IN}> $$str" ; \
	done
.    if defined(PKG_FAIL_REASON)
	@${FALSE}
.    endif
.endif

.PHONY: all
.if !target(all)
.  if ${PKGSRC_RUN_TEST} == "YES" || ${PKGSRC_RUN_TEST} == "yes"
all: test
.  else
all: build
.  endif
.endif

######################################################################
# RULE SETUP
######################################################################

# Filter that removes the last line:
FILTER_BUTLAST=		${AWK} '{ if (have_line) print line; line=$$0; have_line=1 }'

## GODI does not need superuser privileges for installation. So _SU_TARGET
## simply starts the selected action.
## (In NetBSD, _SU_TARGET runs 'su', and executes the $$realtarget with
## superuser privileges.)

_SU_TARGET=                                                             \
		args="";						\
		if [ "X${FORCE_PKG_REGISTER}" != X"" ]; then		\
			args="FORCE_PKG_REGISTER=1";			\
		fi;							\
		if [ "X${PKG_DEBUG_LEVEL}" != X"" ]; then		\
			args="$$args PKG_DEBUG_LEVEL=${PKG_DEBUG_LEVEL}"; \
		fi;							\
		${SETENV} PATH=${PATH:Q}:${SU_CMD_PATH_APPEND:Q} ${MAKE} $$args ${MAKEFLAGS} $$realtarget $$realflags

PKGCONFIG_OVERRIDE_SED=	\
		'-e s|^\(Libs:.*[ 	]\)-L\([ 	]*[^ 	]*\)\(.*\)$$|\1-Wl,${ELF_RPATH_FLAG}\2 -L\2\3|'


MESSAGE_SUBST_SED=	${MESSAGE_SUBST:S/=/}!/:S/$/!g/:S/^/ -e s!\\\${/}


################################################################
# The following are used to create easy dummy targets for
# disabling some bit of default target behavior you don't want.
# They still check to see if the target exists, and if so don't
# do anything, since you might want to set this globally for a
# group of packages in a Makefile.inc, but still be able to
# override from an individual Makefile.
################################################################

# Disable checksum
.PHONY: checksum
.if ((defined(NO_CHECKSUM) || exists(${EXTRACT_COOKIE})) && !target(checksum))
checksum: fetch
	@${DO_NADA}
.endif

# Disable configure
.PHONY: configure
.if defined(NO_CONFIGURE) && !target(configure)
configure: buildlink
	${_PKG_SILENT}${_PKG_DEBUG}${TOUCH} ${TOUCH_FLAGS} ${CONFIGURE_COOKIE:Q}
.endif

# Disable build
.PHONY: build
.if defined(NO_BUILD) && !target(build)
build: configure
	${_PKG_SILENT}${_PKG_DEBUG}${TOUCH} ${TOUCH_FLAGS} ${BUILD_COOKIE:Q}
.endif

# Disable install
.PHONY: install
.if defined(NO_INSTALL) && !target(install)
install: build
	${_PKG_SILENT}${_PKG_DEBUG}${TOUCH} ${TOUCH_FLAGS} ${INSTALL_COOKIE:Q}
.endif

# Disable package
.PHONY: package
.if defined(NO_PACKAGE) && !target(package)
package:
.  if defined(SKIP_SILENT)
	@${DO_NADA}
.  else
	@${ECHO_MSG} "${_PKGSRC_IN}> ${PKGNAME} may not be packaged: ${NO_PACKAGE}."
.  endif
.endif

######################################################################
# SUBSTITUTIONS
######################################################################

.PHONY: message
message: ${MESSAGE}
.ifdef MESSAGE
${MESSAGE}: ${MESSAGE_SRC}
	${_PKG_SILENT}${_PKG_DEBUG}					\
	if [ -z "${MESSAGE_SRC}" ]; then				\
		${ERROR_MSG} "${MESSAGE_SRC} not found.";		\
		${ERROR_MSG} "Please set MESSAGE_SRC correctly.";       \
	else								\
		${CAT} ${MESSAGE_SRC:Q} |				\
			${SED} ${MESSAGE_SUBST_SED}			\
			> ${MESSAGE:Q};					\
	fi
.endif


# generate ${PLIST} from ${PLIST_SRC} by doing the substitutions of
# ${PLIST_SUBST}

.PHONY: plist
plist: ${PLIST}
${PLIST}: ${PLIST_SRC}
	${_PKG_SILENT}${_PKG_DEBUG}                                    \
	${CAT} ${PLIST_SRC:Q} |                                        \
	${SED}  ${PLIST_SUBST:S/=/}!/:S/$/!g/:S/^/ -e s!\\\${/}        \
		> ${PLIST:Q}


# generate ${DESCR} from ${DESCR_SRC} by:
# - Appending the homepage URL, if any

.PHONY: descr
descr: ${DESCR}
${DESCR}: ${DESCR_SRC}
	@${CAT} ${DESCR_SRC:Q} 	 >${DESCR:Q}
.if defined(HOMEPAGE)
	@\
	${ECHO}			>>${DESCR:Q} ; \
	${ECHO} "Homepage:"	>>${DESCR:Q} ; \
	${ECHO} '${HOMEPAGE}'	>>${DESCR:Q}
.endif

######################################################################
# DOWNLOAD
######################################################################

# usage:
#   verify_file "$DISTINFO_FILE" "$path" "$name_as_in_distinfo"
# exit codes:
#   0 = ok
#   10 = no distinfo file
#   11 = no checksum in distinfo file
#   12 = bad checksum

VERIFY_FN_IMPORT =
.text
.expand
${ESSENTIAL_PROGRAMS}
DIGEST
DIGEST_ALGORITHM
.endtext

VERIFY_FN =
.text
verify_file() {
    di_file="$1"
    vfy_file="$2"
    vfy_bfile=`basename "$3"`

    if [ -f "$di_file" ]; then
	alg=`
          ${AWK} -v file="$vfy_bfile" \
            'NF == 4 && $2 == "(" file ")" && $3 == "=" {print $1;}' \
            "$di_file"`
	if [ -z "$alg" ]; then
	    alg="$DIGEST_ALGORITHM"
	fi
	cksum=`$DIGEST "$alg" < "$vfy_file"`
	cksum2=`
          ${AWK} -v alg="$alg" -v file="$vfy_bfile" \
            '$1 == alg && $2 == "(" file ")" {print $4;}'  \
            "$di_file"`
	if [ -z "$cksum2" ]; then
	    return 11
	elif [ "$cksum" = "$cksum2" -o "$cksum2" = "IGNORE" ]; then
	    return 0
	else
	    return 12
	fi
    else
	return 10
    fi
}
.endtext



# usage:
#   fetch_file "$file" "$sites"
# needs verify_file!

FETCH_FN_IMPORT=
.text
.expand
${ESSENTIAL_PROGRAMS}
DISTDIR
DISTINFO_FILE
DISTPREFIX
DISTPREFIX_IN_DISTINFO
FETCH_CMD
FETCH_BEFORE_ARGS
FETCH_AFTER_ARGS
ECHO_MSG
ERROR_MSG
PKGDIR
.endtext

FETCH_FN=
.text
fetch_file() {
    file="$1"
    sites="$2"
    urls="$3"
    bfile=`basename "$file"`
    pfile="${DISTPREFIX}${bfile}"
    ifile="${DISTPREFIX_IN_DISTINFO}${bfile}"
    if [ ! -f "$DISTDIR/$pfile" ]; then
	$ECHO_MSG "=> $bfile doesn't exist on this system."
	if [ ! -w $DISTDIR/. ]; then
	    $ERROR_MSG "=> Cannot download to $DISTDIR (permission denied?)"
	    return 1
	fi
	source=`cat "$PKGDIR"/.source 2>/dev/null || true`
	source=`echo "$source" | tr -d '\r'`
        for site in $sites; do
	    right_source=0
	    case "$site" in
	        {*}*)
		    site_source=`echo "$site" | $SED -e 's/^{\(.*\)}.*$/\1/'`
		    site_url=`echo "$site" | $SED -e 's/^{.*}\(.*\)$/\1/'`
		    if [ "X$source" = "X$site_source" -o -z "$source" ]; then
			right_source=1
		    fi ;;
	        *)
		    site_url="$site"
		    right_source=1 ;;
	    esac
	    if [ $right_source -gt 0 ]; then
		$ECHO_MSG "=> Attempting to fetch $bfile from $site_url."; \
		$FETCH_CMD -O "$pfile" $FETCH_BEFORE_ARGS "${site_url}${bfile}" $FETCH_AFTER_ARGS || continue
		if [ -f "$DISTDIR/$pfile" ]; then
		    good=1
		    verify_file "$DISTINFO_FILE" "$DISTDIR/$pfile" "$ifile" || {
			case "$?" in
			    10)
				$ECHO_MSG "=> Missing distinfo file"
				;;
			    11)
				$ECHO_MSG "=> Missing checksum in distinfo"
				good=0 ;;
			    12)
				$ECHO_MSG "=> Checksum failure"
				good=0 ;;
			    *)
				$ECHO_MSG "=> Verification error"
				good=0 ;;
			esac
		    }
		    if [ $good -gt 0 ]; then break; fi
	        fi
	        $RM -f "$DISTDIR/$pfile"
		$ECHO_MSG "=> Wrong file, trying next site"
	    fi
	done
        for url in $urls; do
	    $ECHO_MSG "=> Attempting to fetch $bfile from $url."; \
	    $FETCH_CMD -O "$pfile" $FETCH_BEFORE_ARGS "${url}" $FETCH_AFTER_ARGS || continue
   	    if [ -f "$DISTDIR/$pfile" ]; then
                good=1
                verify_file "$DISTINFO_FILE" "$DISTDIR/$pfile" "$ifile" || {
                    case "$?" in
                        10)
                            $ECHO_MSG "=> Missing distinfo file"
                            ;;
                        11)
                            $ECHO_MSG "=> Missing checksum in distinfo"
                            good=0 ;;
                        12)
                            $ECHO_MSG "=> Checksum failure"
                            good=0 ;;
                        *)
                            $ECHO_MSG "=> Verification error"
                            good=0 ;;
                    esac
                }
                if [ $good -gt 0 ]; then break; fi
            fi
            $RM -f "$DISTDIR/$pfile"
            $ECHO_MSG "=> Wrong file, trying next site"
        done
	if [ ! -f "$DISTDIR/$pfile" ]; then \
	    $ERROR_MSG "=> Couldn't fetch $bfile - please try to retrieve this"
	    $ERROR_MSG "=> file manually into $DISTDIR and try again."
	    return 1
	fi
    fi
}
.endtext


.PHONY: do-fetch
.if !target(do-fetch)
do-fetch:
.  if !empty(ALLFILES)
.    script
.      import ${FETCH_FN_IMPORT}
.      import ${VERIFY_FN_IMPORT}
.      expand
         ${_PKG_SILENT}${_PKG_DEBUG}
	 set -e
	 ${VERIFY_FN}
         ${FETCH_FN}
	 ${TEST} -d ${DISTDIR:Q} || ${MKDIR} ${DISTDIR:Q}
         for fetchfile in ${ALLFILES:QL}; do
	   cd ${DISTDIR:Q}
	   fetch_file $${fetchfile} ${ALLSITES:QQ} ${MASTER_URLS:QQ}
	 done
.    endscript
.  endif # !empty(ALLFILES)
.endif

.ifmake fetch-master
.PHONY: fetch-master
fetch-master:
.    script
.      import ${FETCH_FN_IMPORT}
.      import ${VERIFY_FN_IMPORT}
.      expand
         ${_PKG_SILENT}${_PKG_DEBUG}
	 set -e
	 ${VERIFY_FN}
         ${FETCH_FN}
	 ${TEST} -d ${DISTDIR:Q} || ${MKDIR} ${DISTDIR:Q}
         for fetchfile in ${ALLFILES:QL}; do
	   cd ${DISTDIR:Q}
	   fetch_file $${fetchfile} ${MASTER_SITES:QQ} ${MASTER_URLS:QQ}
	 done
.    endscript
.endif


.PHONY: show-distfiles
.if !target(show-distfiles)
show-distfiles:
.  if defined(PKG_FAIL_REASON)
	${_PKG_SILENT}${_PKG_DEBUG}${DO_NADA}
.  else
	${_PKG_SILENT}${_PKG_DEBUG}					\
	for file in "" ${CKSUMFILES}; do				\
		if [ "X$$file" = "X" ]; then continue; fi;		\
		${ECHO} "${DISTPREFIX}$$file";					\
	done
.  endif
.endif

.PHONY: checksum
.if !target(checksum)
checksum: fetch
.script
.  import ${VERIFY_FN_IMPORT}
.  import CKSUMFILES
.  import IGNOREFILES
.  import DISTPREFIX
.  import DISTPREFIX_IN_DISTINFO
.  expand
	${_PKG_SILENT}${_PKG_DEBUG}
	set -e
        ${VERIFY_FN}
	if [ ! -f ${DISTINFO_FILE:Q} ]; then
	    ${ECHO_MSG} "=> No distinfo file."
	else
            good=1
	    for file in "" $$CKSUMFILES; do
		if [ "X$$file" = X"" ]; then continue; fi
		verify_file ${DISTINFO_FILE:Q} ${DISTDIR:Q}/${DISTPREFIX:Q}$$file ${DISTPREFIX_IN_DISTINFO:Q}$$file || {
		    case "$$?" in
			10)
			    ${ECHO_MSG} "=> Missing distinfo file"
			    ;;
			11)
			    ${ECHO_MSG} "=> Missing checksum in distinfo for ${DISTPREFIX}$$file"
			    good=0 ;;
			12)
			    ${ECHO_MSG} "=> Checksum failure for ${DISTPREFIX}$$file"
			    good=0 ;;
			*)
			    ${ECHO_MSG} "=> Verification error for ${DISTPREFIX}$$file"
			    good=0 ;;
		    esac
		}
	    done
	    for file in "" $$IGNOREFILES; do
		  if [ "X$$file" = X"" ]; then continue; fi
		  cksum2=`
        ${AWK} -v file="${DISTPREFIX_IN_DISTINFO}$$file" \
          'NF == 4 && $$3 == "=" && $$2 == "(" file ")"{print $$4;}' \
          ${DISTINFO_FILE:Q}`
		  if [ -n "$$cksum2" ]; then
		      ${ECHO_MSG} "=> No checksum recorded for ${DISTPREFIX}$$file, file is in "'$$'"{IGNOREFILES} list."
		      good=0
		  elif [ "X$$cksum2" != "XIGNORE" ]; then
	              ${ECHO_MSG} "=> Checksum for ${DISTPREFIX}$$file is not set to IGNORE in checksum file even though"
		      ${ECHO_MSG} "   the file is in the "'$$'"{IGNOREFILES} list."
		      good=0
		  fi
	    done
	    if [ $$good -eq 0 ]; then
		${ERROR_MSG} "Make sure the Makefile and checksum file (${DISTINFO_FILE})"
		${ERROR_MSG} "are up to date.  If you want to override this check, type"
		${ERROR_MSG} "\"${MAKE} NO_CHECKSUM=yes [other args]\"."
		exit 1
	     fi
	fi
.endscript
.endif


######################################################################
# EXTRACT
######################################################################

# pkgsrc coarse-grained locking definitions and targets

${WRKDIR}:
	${_PKG_SILENT}${_PKG_DEBUG}${MKDIR} ${WRKDIR:Q}
.ifdef WRKOBJDIR
	${_PKG_SILENT}${_PKG_DEBUG}					\
	if ${LN} -s ${WRKDIR:Q} ${WRKDIR_BASENAME:Q} 2>/dev/null; then	\
		${ECHO} "${WRKDIR_BASENAME} -> ${WRKDIR}";		\
	fi
.endif # WRKOBJDIR

_EXTRACT_SUFFICES=	.tar.gz .tgz .tar.bz2 .tbz .tar.Z .tar
_EXTRACT_SUFFICES+=	.shar.gz .shar.bz2 .shar.Z .shar
_EXTRACT_SUFFICES+=	.Z .bz2 .gz .tar.xz .txz .xz .lzma

## GODI: We do not support zip, lha, zoo

DECOMPRESS_CMD.tar.gz=		${GZCAT}
DECOMPRESS_CMD.tgz=		${DECOMPRESS_CMD.tar.gz}
DECOMPRESS_CMD.tar.bz2=	 	${BZCAT}
DECOMPRESS_CMD.tbz=		${DECOMPRESS_CMD.tar.bz2}
#FIXME: integrate in configure script instead of hardcoded path
DECOMPRESS_CMD.tar.xz=	 	/bin/xzcat
DECOMPRESS_CMD.txz=		${DECOMPRESS_CMD.tar.xz}
DECOMPRESS_CMD.tar.lzma=	/bin/lzcat
DECOMPRESS_CMD.tar.Z=		${GZCAT}
DECOMPRESS_CMD.tar=		${CAT}

DECOMPRESS_CMD.shar.gz=		${GZCAT}
DECOMPRESS_CMD.shar.bz2=	${BZCAT}
DECOMPRESS_CMD.shar.Z=		${GZCAT}
DECOMPRESS_CMD.shar=		${CAT}

DECOMPRESS_CMD.Z=		${GZCAT}
DECOMPRESS_CMD.bz2=		${BZCAT}
DECOMPRESS_CMD.gz=		${GZCAT}
DECOMPRESS_CMD.lzma=		/bin/lzcat
DECOMPRESS_CMD.xz=		/bin/xzcat

.undef EXTRACT_CMD.tar.gz
.undef EXTRACT_CMD.tgz
.undef EXTRACT_CMD.tar.bz2
.undef EXTRACT_CMD.tbz
.undef EXTRACT_CMD.tar.Z
.undef EXTRACT_CMD.tar

.undef EXTRACT_CMD.shar.gz
.undef EXTRACT_CMD.shar.bz2
.undef EXTRACT_CMD.shar.Z
.undef EXTRACT_CMD.shar

.undef EXTRACT_CMD.Z
.undef EXTRACT_CMD.bz2
.undef EXTRACT_CMD.gz


DOWNLOADED_DISTFILE=	$${extract_file}

.if !defined(W32PORT) || ${W32PORT} != "mingw"

.for __suffix__ in .gz .bz2 .Z
EXTRACT_CMD${__suffix__}=	${DECOMPRESS_CMD${__suffix__}} "$${extract_file}" > `${BASENAME} "$${extract_file}" ${__suffix__}`
.endfor

.for __suffix__ in .shar.gz .shar.bz2 .shar.Z .shar
EXTRACT_CMD${__suffix__}=	${DECOMPRESS_CMD${__suffix__}} "$${extract_file}" | ${SH}
.endfor

# do not call gunzip but rely on the integrated gzip support of pax:
PAX_Z = ${PAX} -r -x tar -z
EXTRACT_CMD.tar.gz= 	${PAX_Z} -f "$${extract_file}" ${EXTRACT_ELEMENTS}
EXTRACT_CMD.tgz= 	${PAX_Z} -f "$${extract_file}" ${EXTRACT_ELEMENTS}

# Use pax in preference to (GNU) tar.
#
# Quoting BUG: ${EXTRACT_ELEMENTS}
_DFLT_EXTRACT_CMD=	${DECOMPRESS_CMD} "$${extract_file}" | ${PAX} -r -x tar ${EXTRACT_ELEMENTS}

.for __suffix__ in ${_EXTRACT_SUFFICES}
.  if !defined(EXTRACT_CMD${__suffix__})
EXTRACT_CMD${__suffix__}=	${DECOMPRESS_CMD${__suffix__}} "$${extract_file}" | ${PAX} -r -x tar ${EXTRACT_ELEMENTS}
.  endif
.endfor

.else

.for __suffix__ in .gz .bz2 .Z .xz
EXTRACT_CMD${__suffix__}=	${DECOMPRESS_CMD${__suffix__}} "$${extract_file}" > `${BASENAME} "$${extract_file}" ${__suffix__}`
.endfor

.for __suffix__ in .shar.gz .shar.bz2 .shar.Z .shar
EXTRACT_CMD${__suffix__}=	${DECOMPRESS_CMD${__suffix__}} "$${extract_file}" | ${SH}
.endfor

_DFLT_EXTRACT_CMD=	${DECOMPRESS_CMD} "$${extract_file}" | /bin/tar --no-same-owner --no-same-permissions -xf- ${EXTRACT_ELEMENTS}

.for __suffix__ in ${_EXTRACT_SUFFICES}
.  if !defined(EXTRACT_CMD${__suffix__})
EXTRACT_CMD${__suffix__}=	${DECOMPRESS_CMD${__suffix__}} "$${extract_file}" | /bin/tar --no-same-owner --no-same-permissions -xf- ${EXTRACT_ELEMENTS}
.  endif
.endfor

.endif



# _SHELL_EXTRACT is a "subroutine" for extracting an archive.  It extracts
# the contents of archive named by the shell variable "extract_file" based
# on the file extension of the archive.
#
_SHELL_EXTRACT=		case "$${extract_file}" in
.for __suffix__ in ${_EXTRACT_SUFFICES}
_SHELL_EXTRACT+=	*${__suffix__})	${EXTRACT_CMD${__suffix__}} ;;
.endfor
_SHELL_EXTRACT+=	*)		${_DFLT_EXTRACT_CMD} ;;
_SHELL_EXTRACT+=	esac

EXTRACT_CMD?=		${_SHELL_EXTRACT}

.PHONY: do-extract
.if !target(do-extract)
do-extract: ${WRKDIR}
.  for __file__ in ${EXTRACT_ONLY}
	${_PKG_SILENT}${_PKG_DEBUG}					\
	extract_file=${DISTDIR:Q}/${__file__:Q}; export extract_file;	\
	cd ${WRKDIR:Q}; ${EXTRACT_CMD}
.if defined(DISTNAME) && ${DISTNAME} != "" && ${DISTNAME} != "." && ${DISTNAME} != "./" && defined(W32PORT) && ${W32PORT} == "mingw"
	@cd ${WRKDIR:Q} && ${LOCALBASE}/lib/godi/replacesymlinks.sh ${DISTNAME:Q}
.endif
.  endfor
.  if defined(DISTNAME) && !empty(DISTNAME)
	${MKDIR} ${WRKSRC}
.  endif
.endif

######################################################################
# PATCH
######################################################################

# LOCALPATCHES contains the location of local patches to packages
#	that are maintained in a directory tree reflecting the same
#	hierarchy as the pkgsrc tree, i.e. local patches for www/apache
#	would be found as ${LOCALPATCHES}/www/apache/*.
#
.if defined(LOCALPATCHES)
_LOCALPATCHFILES=	${LOCALPATCHES}/${PKGPATH}
.endif

.if defined(PROFILEPATCHES)
_PROFILEPATCHFILES=	${PROFILEPATCHES}/${PKGPATH}
.endif

PATCH_SORT_FN=
.text
patch_sort() {
    echo "$@" | $SED -e '/^.*-\*/ d' | $SED -e 's:^\(.*/[^/-]*\)-\([^/]*\)$:\1-\2 \2:' | LC_ALL=C $SORT -k 2 | $SED -e 's/^\([^ ]\+\) .*$/\1/'
}
.endtext


PATCH_IMPORT=
.text
.expand
${ESSENTIAL_PROGRAMS}
DISTDIR
DISTINFO_FILE
ECHO_MSG
ERROR_MSG
PKGNAME
PKGDIR
PATCHDIR
LOCALPATCHES
_LOCALPATCHFILES
PROFILEPATCHES
_PROFILEPATCHFILES
_PKGSRC_IN
PATCH_FUZZ_FACTOR
PATCH_ARGS
PATCH_VERSION_FILE
WRKSRC
DIGEST_ALGORITHM
.endtext


.PHONY: do-patch
.if !target(do-patch)
do-patch:
.script
.  import ${PATCH_IMPORT}
.  import ${VERIFY_FN_IMPORT}
.  expand
	${_PKG_SILENT}${_PKG_DEBUG}
	${VERIFY_FN}
	${PATCH_SORT_FN}
.  noexpand
        set -e
	${RM} -f "${PATCH_VERSION_FILE}"
	${TOUCH} "${PATCH_VERSION_FILE}"
	vfy_patchlist=""
	patchlist=""
	if [ -d "$PATCHDIR" ]; then
	    vfy_patchlist="$(patch_sort "$($LS "$PATCHDIR"/file-* "$PATCHDIR"/patch-* 2>/dev/null)")"
	fi
        if [ -n "$_PROFILEPATCHFILES" ]; then
	    patchlist="$(patch_sort "$($LS "$_PROFILEPATCHFILES"/file-* "$_PROFILEPATCHFILES"/patch-* 2>/dev/null)")"
        fi
        if [ -n "$_LOCALPATCHFILES" ]; then
	    files="$(patch_sort "$($LS "$_LOCALPATCHFILES"/file-* "$_LOCALPATCHFILES"/patch-* 2>/dev/null)")"
	    patchlist="$patchlist $files"
        fi
	if [ -n "$vfy_patchlist" -o -n "$patchlist" ]; then
	    $ECHO_MSG "${_PKGSRC_IN}> Applying pkgsrc patches for ${PKGNAME}"
	    # only verify the regular patches
	    ok_patchlist=""
	    for i in $vfy_patchlist; do
		if [ ! -f "$i" ]; then
		    continue
		fi
	 	b=`basename "$i"`
		case "$b" in
		    patch-*|file-*)
		        good=1
		        verify_file "$DISTINFO_FILE" "$i" "$b" || {
			    case "$?" in
				10)
				    $ECHO_MSG "=> Missing distinfo file"
				    ;;
				11)
				    $ECHO_MSG "=> Missing checksum in distinfo"
				    good=0 ;;
				12)
				    $ECHO_MSG "=> Checksum failure"
				    good=0 ;;
				*)
				    $ECHO_MSG "=> Verification error"
				    good=0 ;;
			    esac
			}
			if [ $good -eq 0 ]; then
			    ${ECHO_MSG} "**************************************"
			    ${ECHO_MSG} "Ignoring bad patch file: $i"
			    ${ECHO_MSG} "**************************************"
			    continue
			fi
			ok_patchlist="$ok_patchlist $i"
			;;
		esac
	    done
	    # regular patches come first
	    patchlist="$ok_patchlist $patchlist"
	    cd "$WRKSRC"
	    for i in $patchlist; do
		if [ ! -f "$i" ]; then
		    $ECHO_MSG "${_PKGSRC_IN}> $i is not a valid patch file - skipping"
		    continue
		fi
	 	b=`basename "$i"`
		${ECHO_MSG} "${_PKGSRC_IN}> Applying $i"
		case "$b" in
		    patch-*)
			fuzz=""
		        $PATCH -v > /dev/null 2>&1 && fuzz="$PATCH_FUZZ_FACTOR"
			$PATCH $fuzz $PATCH_ARGS < "$i" ||
			  { $ECHO Patch "$i" failed ; exit 1; }
			$DIGEST -strip "$PATCHDIR/" $DIGEST_ALGORITHM "$i" \
			   >>"$PATCH_VERSION_FILE"
			;;
		    file-*)
			{ while read cmd; do
			      case "$cmd" in
				  add:\ *)
				      n=`echo "$cmd" | sed -e 's/^add: \(.*\)$/\1/'`
				      read lines
				      $ECHO_MSG "adding $n"
				      $HEAD -n "$lines" > "$WRKSRC/$n"
				      ;;
				  del:\ *)
				      n=`echo "$cmd" | sed -e 's/^del: \(.*\)$/\1/'`
				      $ECHO_MSG "removing $n"
				      $RM "$WRKSRC/$n"
				      ;;
			      esac
			  done
			} < "$i"
			$DIGEST -strip "$PATCHDIR/" $DIGEST_ALGORITHM "$i" \
			   >>"$PATCH_VERSION_FILE"
			;;
		    *)
			$ECHO_MSG "ignoring $i"
			;;
		esac
	    done
	fi
.endscript
.endif


INCR_PATCH_FN=
.text
incr_patch() {
    tmpx="$(mktemp)"
    cat - >"$tmpx"  <<'EOF'
(* --- ocaml script --- *)
let p = Sys.argv.(1);;
let c1 =
  if String.length p < 2 then
    '`'
  else
    p.[1];;
let c1' = Char.chr (Char.code c1 + 1);;
let c0 =
  if String.length p < 1 then
    '`'
  else
    p.[0];;
let c0' = Char.chr (Char.code c0 + 1);;
let q = String.make 2 ' ';;
if c1 = '{' then (
  q.[1] <- 'a';
  q.[0] <- c0'
)
else (
  q.[1] <- c1';
  q.[0] <- c0
);;
print_endline q;;
(* --- END ocaml script '` --- *)
EOF
  ocaml -w -A "$(cygpath -m "$tmpx")" "$1"
  rm "$tmpx"
}
.endtext


DOCUMENT_FILES?=
DOCUMENT_FILES_IGNORE?=
DOCUMENT_DIRS?=
IMPORT_FILES?=
DOCUMENT_NO_AUTO?=0
.PHONY: destdir-copy-docs
.if !target(destdir-copy-docs)
destdir-copy-docs:
.script
.	import LOCALBASE WRKSRC PKGBASE IMAGE_DIR INSTALL
.	import FILE_CMD TR MKDIR BASENAME DIRNAME PAX
.	import DOCUMENT_FILES DOCUMENT_FILES_IGNORE DOCUMENT_DIRS DOCUMENT_NO_AUTO
.	import IMPORT_FILES MV
.	expand
		${_PKG_SILENT}${_PKG_DEBUG}
.	noexpand
set -e
set -u

if [ -n "${IMPORT_FILES}" ]; then
  cd "${LOCALBASE}"
  for f in ${IMPORT_FILES}; do
    if [ -z "$f" ] || [ ! -e "$f" ] ; then
      continue
    fi
    aim="${IMAGE_DIR}${LOCALBASE}"
    dirname="$($DIRNAME "$f")"
    basename="$($BASENAME "$f")"
    if [ -n "$dirname" ] && [ "$dirname" != "." ] ; then
      aim="${aim}/${dirname}"
      ${MKDIR} "$aim"
      cd "$dirname"
    fi
    ${MV} "$basename" "$aim"
    cd "${LOCALBASE}"
  done
fi

doc_dir="${IMAGE_DIR}${LOCALBASE}/doc/${PKGBASE}"
if [ ! -d "$doc_dir" ]; then
    ${MKDIR} "$doc_dir"
fi
cd "$WRKSRC"

if [ -n "${DOCUMENT_DIRS}" ]; then
    for d in "${DOCUMENT_DIRS}" ; do
        if [ -z "$d" ] || [ ! -d "$d" ] ; then
            exit 1
        fi
        dirname="$($DIRNAME "$d")"
        basename="$($BASENAME "$d")"
        cd "$dirname"
        ${PAX} -rw -pp "$basename" "${doc_dir}"
        cd "$WRKSRC"
    done
fi

if [ -n "${DOCUMENT_FILES}" ]; then
    for f in ${DOCUMENT_FILES} ; do
        ${INSTALL} -m 0444 "$f" "${doc_dir}"
    done
fi

if [ -n "$DOCUMENT_NO_AUTO" ] ; then
    case "$DOCUMENT_NO_AUTO" in
        1*) exit 0 ;;
        Y*) exit 0 ;;
        y*) exit 0 ;;
        t*) exit 0 ;;
        T*) exit 0 ;;
    esac
fi

do_install(){
    local do_copy
    do_copy=1
    if [ -n "${DOCUMENT_FILES_IGNORE}" ]; then
        for f in ${DOCUMENT_FILES_IGNORE} ; do
            if [ "$f" = "$1" ]; then
                do_copy=0
                break
            fi
        done
    fi
    if [ ${do_copy} -eq 1 ] && [ -f "$1" ]; then
        dst="${doc_dir}/$1"
        if [ ! -e "$dst" ]; then
            ${INSTALL} -m 0444 "$1" "$dst"
        fi
    fi
}

for file in *.[Mm][Dd] *.[Hh][Tt][Mm] *.[Hh][Tt][Mm][Ll] *.[Cc][Ss][Ss] \
    [Aa][Uu][Tt][Hh][Oo][Rr][Ss] [Rr][Ee][Aa][Dd][Mm][Ee] [Ll][Ii][Cc][Ee][Nn][Ss][Ee] [Tt][Oo][Dd][Oo] \
    [Cc][Hh][Aa][Nn][Gg][Ee][Ss] [Cc][Oo][Pp][Yy][Ii][Nn][Gg] [Cc][Hh][Aa][Nn][Gg][Ee][Ll][Oo][Gg] \
    [Vv][Ee][Rr][Ss][Ii][Oo][Nn] [Gg][Pp][Ll] [Ll][Gg][Pp][Ll] [Mm][Ii][Tt] ; do
    if [ -f "$file" ] ; then
        do_install "$file"
    fi
done

for file in *.[Tt][Xx][Tt] ; do
    case "$file" in
        *[Cc][Mm][Aa][Kk][Ee]*) continue ;;
    esac
    do_install "$file"
done

for file in *.[Pp][Dd][Ff] ; do
    case "$file" in
        *.[0-9].[Pp][Dd][Ff])
            continue ;; # man pages as pdf
        *) do_install "$file" ;;
    esac
done

for file in * ; do
    ok=0
    case "$file" in
        _*) continue ;;
        .*) continue ;;
        \#*) continue ;;
        *~) continue ;;
        # skip well known source files and build artefacts
        *.[Aa][Ww][Kk]) continue ;;
        *.[Bb][Aa][Tt]) continue ;;
        *.[Cc][Nn][Ff]) continue ;;
        *.[Cc][Oo][Mm]) continue ;;
        *.[Cc][Oo][Nn][Ff]) continue ;;
        *.[Cc][Oo][Nn][Ff][Ii][Gg]) continue ;;
        *.[Dd]) continue ;;
        *.[Dd][Oo][Aa][Pp]) continue ;;
        *.[Dd][Oo][Xx][Yy]) continue ;;
        *.[Ll][Aa]) continue ;;
        *.[Ll][Oo]) continue ;;
        *.[Pp][Oo]) continue ;;
        *.[Ss]) continue ;;
        *.[Ss][Tt][Aa][Mm][Pp]) continue ;;
        *.[Ss][Yy][Mm][Ss]) continue ;;
        *.[0-9]) continue ;;
        *.[Aa][Mm]) continue ;;
        *.[Bb][Aa][Kk]) continue ;;
        *.[Cc]) continue ;;
        *.[Cc][Cc]) continue ;;
        *.[Cc][Ff][Gg]) continue ;;
        *.[Cc][Mm][Aa][Kk]*) continue ;;
        *.[Cc][Nn][Ff]) continue ;;
        *.[Cc][Oo][Nn][Ff]) continue ;;
        *.[Cc][Pp][Pp]) continue ;;
        *.[Dd][Ee][Ff]) continue ;;
        *.[Dd][Ee][Pp]) continue ;;
        *.[Dd][Ss][Pp]) continue ;;
        *.[Dd][Ss][Ww]) continue ;;
        *.[Dd][Tt][Dd]) continue ;;
        *.[Hh]) continue ;;
        *.[Hh][Pp][Pp]) continue ;;
        *.[Ii][Nn]) continue ;;
        *.[Ll][Ss][Mm]) continue ;;
        *.[Mm]4) continue ;;
        *.[Mm][Aa][Kk]) continue ;;
        *.[Mm][Aa][Kk][Ee]) continue ;;
        *.[Mm][Kk]) continue ;;
        *.[Mm][Ll]) continue ;;
        *.[Mm][Ll][Ii]) continue ;;
        *.[Oo][Rr][Ii][Gg]) continue ;;
        *.[Pp][Aa][Ss]) continue ;;
        *.[Pp][Cc]) continue ;;
        *.[Pp][Ll]) continue ;;
        *.[Pp][Ss]) continue ;;
        *.[Rr][Cc]) continue ;;
        *.[Rr][Ee][Ff]) continue ;;
        *.[Ss][Hh]) continue ;;
        *.[Ss][Pp][Ee][Cc]) continue ;;
        *.[Tt][Ee][Xx]) continue ;;
        *.[Tt][Ee][Xx][Ii][Nn][Ff][Oo]) continue ;;
        *.[Vv][Cc][Ww]) continue ;;
        *.[Xx][Mm][Ll]) continue ;;
        *.[Xx][Ss][Ll]) continue ;;
        *-[Dd][Ee][Pp][Ss]) continue ;;
        *-[Dd][Ee][Pp]) continue ;;
        ABOUT-NLS) continue ;;
        Doxyfile) continue ;;
        META) continue ;;
	[Oo][Pp][Aa][Mm] ) continue ;;
        Jam*) continue ;;
        [Bb][Oo][Oo][Tt][Ss][Tt][Rr][Aa][Pp]*) continue ;;
        [Cc][Oo][Nn][Ff][Ii][Gg]*) continue ;;
        [Dd][Ee][Pp]) continue ;;
        [Dd][Ee][Pp][Cc][Oo]*) continue ;;
        [Dd][Ee][Pp][Ee][Nn][Dd]*) continue ;;
        [Ii][Nn][Ss][Tt][Aa][Ll][Ll]*) continue ;;
        stamp-*) continue ;;
        words[0-9]) continue ;;
        *CMake*) continue ;;
        *[Mm][Aa][Kk][Ee][Ff][Ii][Ll][Ee]*) continue ;;
        *[Mm][Kk][Ff][Ii]*) continue;;
        *libtool*) continue ;;
        *mapfile*) continue ;;

        # copy everything what looks like a license
        *[Aa][Uu][Tt][Hh][Oo][Rr]*) ok=1 ;;
        *[Cc][Hh][Aa][Nn][Gg]*) ok=1 ;;
        *[Cc][Oo][Pp][Yy]*) ok=1 ;;
        *[Ll][Ii][Cc][Ee]*) ok=1 ;;
        *[Rr][Ee][Aa][Dd][Mm][Ee]*) ok=1 ;;
        *[Dd][Ii][Ss][Cc][Ll][Aa][Ii][Mm]*) ok=1 ;;
        *[Tt][Hh][Ii][Rr][Dd]-[Pp][Aa][Rr][Tt]*) ok=1 ;;
        *[Gg][Pp][Ll]*) ok=1 ;;
        *[Bb][Oo][Oo][Ss][Tt]*) ok=1 ;;
        *[Ff][Aa][Qq]*) ok=1 ;;
        *[Tt][Oo][Dd][Oo]*) ok=1 ;;
        *[Hh][Aa][Cc][Kk][Ii][Nn][Gg]*) ok=1 ;;
        *[Nn][Oo][Tt][Ee]*) ok=1 ;;
        *[Rr][Ee][Ll][Ee][Aa][Ss][Ee]*) ok=1 ;;
        *[Mm][Aa][Ii][Nn][Tt][Aa][Ii][Nn]*) ok=1 ;;
        *[Nn][Ee][Ww][Ss]*) ok=1 ;;
        *[Aa][Cc][Kk][Nn][Oo][Ww][Ll][Ee][Dd][Gg][Ee]*) ok=1 ;;
        *[Pp][Rr][Oo][Bb][Ll][Ee][Mm]*) ok=1 ;;
        *[Ff][Ee][Aa][Uu][Tt][Uu][Rr][Ee]*) ok=1 ;;
        *[Hh][Ee][Ll][Pp]*) ok=1 ;;
        *[Ii][Nn][Tt][Rr][Oo]*) ok=1 ;;
        O[Mm]*) continue ;; # omake file
                # enable me later
        *.*) continue ;;
        [A-Za-z]*) ok=1;;
        *) continue ;;
    esac
    if [ $ok -eq 1 ] && [ -f "$file" ] ; then
        LANGUAGE=C
        LANG=C
        LC_ALL=C
        export LC_ALL LANG LANGUAGE
        ftype="$(${FILE_CMD} -b "$file" | $TR '[:upper:]' '[:lower:]')"
        case "$ftype" in
            *sgml*) continue ;;
            *xml*)continue ;;
            *non-iso*) continue ;;
            *libtool*) continue;;
            *' script'*) continue;;
            *'c source'*) continue;;
            *'object file'*) continue ;;
            *'library file'*) continue ;;
            *'very long lines'*) continue ;;
            *' text'*)
                do_install "$file"
        esac
    fi
done
.endscript
.endif


PATCHREF=	${BUILD_DIR}/work-ref

.PHONY: makepatch
makepatch:
.script
.  import ${ESSENTIAL_PROGRAMS} MAKE
.  import PATCHREF WRKDIR WRKSRC DISTNAME SAVE
.  import LOCALPATCHES PROFILEPATCHES PKGPATH BUILD_DIR
.  import ADD_FILES DEL_FILES PKG_TOOLS_BIN
.  expand
	${_PKG_SILENT}${_PKG_DEBUG}
	${INCR_PATCH_FN}
.  noexpand
	set -e
	$RM -rf "$PATCHREF"
	$MKDIR "$PATCHREF"
	echo "### Unpacking reference tree to work-ref"
	$MAKE patch WRKDIR="$PATCHREF"
	echo "### Finding differences"
	( cd "$WRKSRC"; diff -aur "../../work-ref/$DISTNAME" . || true ) | \
	     $GREP '^[+ !@*\-].*' \
	     > "$PATCHREF/diff" 2>/dev/null
	if ! $GREP '^---' "$PATCHREF/diff" >/dev/null; then
	    $RM "$PATCHREF/diff"
	    $TOUCH "$PATCHREF/diff"
	fi
	$TOUCH "$PATCHREF/file"
	if [ -n "$ADD_FILES" ]; then
	    for file in $ADD_FILES; do
	        if [ -f "$WRKSRC/$file" ]; then
		    echo "add: $file" >> "$PATCHREF/file"
		    $CAT "$WRKSRC/$file" | $WC -l >> "$PATCHREF/file"
		    cat "$WRKSRC/$file" >> "$PATCHREF/file"
		else
		    echo "File not found: $file" >&2
		fi
	    done
	fi
	if [ -n "$DEL_FILES" ]; then
	    for file in $ADD_FILES; do
	        if [ -f "$PATCHREF/$DISTFILE/$file" ]; then
		     echo "del: $file" >> "$PATCHREF/file"
		fi
	    done
	fi
	case "X$SAVE" in
		X)
			echo "### Files with differences:"
			$CAT "$PATCHREF/diff"
			echo "### Added/deleted files:"
			if [ -z "$ADD_FILES" -a -z "$DEL_FILES" ]; then
			    echo "(none specified in ADD_FILES and DEL_FILES)"
			fi
			$CAT "$PATCHREF/file"
			echo "### Patch has not been saved!"
			echo "Set SAVE to save (e.g. godi_make makepatch SAVE=...):"
			echo " - SAVE=LOCAL: Save as local patch (in dir LOCALPATCHES)"
			echo " - SAVE=PROFILE: Save as profile patch"
			echo " - SAVE=PACKAGE: Save as package patch"
			exit
			;;
		XLOCAL)
			if [ -z "$LOCALPATCHES" ]; then
			    echo "Var not set: LOCALPATCHES" >&2
			    exit 1
			fi
			patchdir="$LOCALPATCHES/$PKGPATH"
			;;
		XPROFILE)
			patchdir="$PROFILEPATCHES/$PKGPATH"
			;;
		XPACKAGE)
			patchdir="$BUILD_DIR/patches"
			;;
		*)
			echo "Possible SAVE destinations: LOCAL, PROFILE, PACKAGE" >&2
			exit 1
			;;
	esac
        # Determine a patch number
	echo "### Saving patch"
	$MKDIR "$patchdir"
        p="a\`"
	f="$(cd "$patchdir"; $LS -1 | $GREP "^patch-" | $TAIL -n 1 | 
	     $SED -e 's/^patch-//')"
	if expr "x$f" '>' "x$p" >/dev/null; then p="$f"; fi
	f="$(cd "$patchdir"; $LS -1 | $GREP "^file-" | $TAIL -n 1 | 
	     $SED -e 's/^file-//')"
	if expr "x$f" '>' "x$p" >/dev/null; then p="$f"; fi
        # Increase p by 1
	q="$(incr_patch "$p" | tr -d '\r' )"
        # Save patch
	$MKDIR "$patchdir"
	if [ -s "$PATCHREF/diff" ]; then
	    $CP "$PATCHREF/diff" "$patchdir/patch-$q"
	    echo "Wrote patch to $patchdir/patch-$q"
	fi
	if [ -s "$PATCHREF/file" ]; then
	    $CP "$PATCHREF/file" "$patchdir/file-$q"
	    echo "Wrote patch to $patchdir/file-$q"
	fi
        # Register patch
	case "X$SAVE" in
		XLOCAL)
			true
			;;
		XPROFILE)
			${PKG_TOOLS_BIN}/godi_console profile -save-patches
			echo "Saved patch in profile.ini"
			;;
		XPACKAGE)
			$MAKE makepatchsum
			echo "Registered patch in distinfo"
			;;
	esac
        # remove ref
	$RM -rf "$PATCHREF"
.endscript

######################################################################
# CONFIGURE
######################################################################

# _CONFIGURE_PREREQ is a list of targets to run after pre-configure but before
#	do-configure.  These targets typically edit the files used by the
#	do-configure target.  The targets are run as dependencies of
#	pre-configure-override.
#
# _CONFIGURE_POSTREQ is a list of targets to run after do-configure but before
#	post-configure.  These targets typically edit the files generated by
#	the do-configure target that are used during the build phase.

.if defined(PKGCONFIG_OVERRIDE) && ${ELF_RPATH} == "yes"
_CONFIGURE_PREREQ+=	do-pkgconfig-override
.PHONY: do-pkgconfig-override
do-pkgconfig-override:
.  for pkgconfig in ${PKGCONFIG_OVERRIDE}
	${_PKG_SILENT}${_PKG_DEBUG}					\
	if [ -f ${pkgconfig:Q} ]; then					\
		${MV} ${pkgconfig:Q} ${pkgconfig:Q}.norpath ;		\
		${SED} ${PKGCONFIG_OVERRIDE_SED}			\
			< ${pkgconfig:Q}.norpath > ${pkgconfig:Q} ;	\
	fi
.  endfor
.endif


# By adding this target, it makes sure the the above PREREQ's work.
.PHONY: pre-configure-override
pre-configure-override: ${_CONFIGURE_PREREQ}
	@${DO_NADA}

.PHONY: do-configure
.if !target(do-configure)
do-configure:
.  if defined(HAS_CONFIGURE) && ${HAS_CONFIGURE} != "no"
.    for DIR in ${CONFIGURE_DIRS}
.    if ( defined(WINOASIS) && ${WINOASIS} == "yes" ) || ( defined(WINGMAKE) && ${WINGMAKE} == "yes" )
	${_PKG_SILENT}${_PKG_DEBUG}${_ULIMIT_CMD}cd ${DIR:Q} && ${SETENV} \
	    ${CONFIGURE_ENV} ${SANDBOX_CONFIGURE_CMD} \
            ${CONFIGURE_SCRIPT}
.    else
	${_PKG_SILENT}${_PKG_DEBUG}${_ULIMIT_CMD}cd ${DIR:Q} && ${SETENV} \
	    ${CONFIGURE_ENV} ${SANDBOX_CONFIGURE_CMD} \
            ${CONFIGURE_SCRIPT} ${CONFIGURE_ARGS}
.    endif
.    endfor
.  endif
.endif

.PHONY: post-configure
post-configure: ${_CONFIGURE_POSTREQ}

######################################################################
# BUILD
######################################################################

.PHONY: do-build
.if !target(do-build)
do-build:
.  for DIR in ${BUILD_DIRS}
	${_PKG_SILENT}${_PKG_DEBUG}${_ULIMIT_CMD}cd ${DIR:Q} && ${SETENV} ${BUILD_ENV} ${SANDBOX_BUILD_CMD} ${BUILD_SCRIPT}
.  endfor
.endif

######################################################################
# TEST
######################################################################

.PHONY: do-test
.if !target(do-test)
do-test:
.  if defined(TEST_SCRIPT)
.    for DIR in ${TEST_DIRS}
	${_PKG_SILENT}${_PKG_DEBUG}${_ULIMIT_CMD}cd ${DIR:Q} && ${SETENV} ${TEST_ENV} ${SANDBOX_TEST_CMD} ${TEST_SCRIPT}
.    endfor
.  else
	@${DO_NADA}
.  endif
.endif

######################################################################
# INSTALL / REGISTER
######################################################################

.PHONY: do-install
.if !target(do-install)
do-install:
.  for DIR in ${INSTALL_DIRS}
	${_PKG_SILENT}${_PKG_DEBUG}${_ULIMIT_CMD}cd ${DIR:Q} && ${SETENV} ${INSTALL_ENV} ${SANDBOX_INSTALL_CMD} ${INSTALL_SCRIPT}
.  endfor
.endif

# Note about conflicts: The rule real-su-install only checks whether this
# package lists a conflict with another package, but not whether a foreign
# package lists a conflict with this package. The latter is hard to to here.
# Godi_console performs the checks in both directions.

MAKE_INSTALL_TASK=	${MAKE} -C ${.CURDIR:Q} ${MAKEFLAGS} 

MAKE_INSTALL_TASK_IMG=
.text
.expand
OCAMLFIND_DESTDIR=${AUTOGENERATE_IMAGE:Q:n}/lib/ocaml/pkg-lib ${MAKE} ${MAKEFLAGS} PREFIX=${AUTOGENERATE_IMAGE:Q}
.endtext

_OCAMLFIND_QUERY=${SETENV} OCAMLFIND_CONF=${LOCALBASE}/etc/findlib-BUILD.conf \
                 ${LOCALBASE}/bin/ocamlfind query 

.PHONY: do-su-install
do-su-install:
	@${ECHO_MSG} "${_PKGSRC_IN}> Installing for ${PKGNAME}"
	${_PKG_SILENT}${_PKG_DEBUG}					\
	realtarget="real-su-install";					\
	action="install";						\
	${_SU_TARGET}

.PHONY: real-su-install
real-su-install: ${MESSAGE}
.if ${GODI_VERIFY_PLIST} == "YES" || ${GODI_VERIFY_PLIST} == "yes"
	${_PKG_SILENT}${_PKG_DEBUG} ${MAKE_INSTALL_TASK} checkpoint
. endif
.if !defined(NO_PKG_REGISTER) && !defined(FORCE_PKG_REGISTER)
.  if defined(CONFLICTS)
	${_PKG_SILENT}${_PKG_DEBUG} \
	${RM} -f ${WRKDIR:Q}/.CONFLICTS
.    for conflict in ${CONFLICTS}
.      script
.        expand
	   ${_PKG_SILENT}${_PKG_DEBUG}
	   found="`${PKG_DEP} \"${conflict}\" 2>/dev/null || ${TRUE}`"
	   if [ X"$$found" != X"" ]; then
	     ${ECHO} "$$found" >> ${WRKDIR:Q}/.CONFLICTS
	   fi
.       endscript
.     endfor
.     script
.       expand
	${_PKG_SILENT}${_PKG_DEBUG}
	if [ -s ${WRKDIR:Q}/.CONFLICTS ]; then
		found=`${SED}
                   -e s'|${GODI_DBDIR}/||g' 
                     ${WRKDIR:Q}/.CONFLICTS | ${TR} '\012' ' '`
		${ERROR_MSG} \
	          "${_PKGSRC_IN}> ${PKGNAME} conflicts with installed package(s): $$found found."
		${ERROR_MSG} \
                  "*** They install the same files into the same place."
		${ERROR_MSG} \
                  "*** Please remove $$found first with godi_delete(1)."
		${RM} -f ${WRKDIR:Q}/.CONFLICTS
		exit 1
	fi
.     endscript
.  endif	# CONFLICTS
.  if defined(PROVIDE_FINDLIB) && exists(${LOCALBASE}/bin/ocamlfind)
	${_PKG_SILENT}${_PKG_DEBUG} \
	${RM} -f ${WRKDIR:Q}/.CONFLICTS
.    for lib in ${PROVIDE_FINDLIB}
.      script
.        expand
	   ${_PKG_SILENT}${_PKG_DEBUG}
	   found="`${_OCAMLFIND_QUERY} \"${lib}\" 2>/dev/null || ${TRUE}`"
	   if [ X"$$found" != X"" ]; then
	     ${ECHO} "$$found" >> ${WRKDIR:Q}/.CONFLICTS
	   fi
.      endscript
.      script
.        expand
	 ${_PKG_SILENT}${_PKG_DEBUG}
	 if [ -s ${WRKDIR:Q}/.CONFLICTS ]; then
		found=`${CAT} ${WRKDIR:Q}/.CONFLICTS`
		${ERROR_MSG} \
	          "${_PKGSRC_IN}> ${PKGNAME} conflicts with installed libraries: $$found"
		${ERROR_MSG} \
                  "*** The package would install the same libraries."
		${RM} -f ${WRKDIR:Q}/.CONFLICTS
		exit 1
	 fi
.      endscript
.    endfor
.  endif        # PROVIDE_FINDLIB
.  script
.     expand
      ${_PKG_SILENT}${_PKG_DEBUG}
      found="`${PKG_DEP} \"${PKGWILDCARD}\" 2>/dev/null || ${TRUE}`"
      if [ "$$found" != "" ]; then
	      ${ERROR_MSG} \
                "${_PKGSRC_IN}>  $$found is already installed - perhaps an older version?"
	      exit 1
      fi
.  endscript
.endif # !NO_PKG_REGISTER && !NO_FORCE_REGISTER
.script
.  expand
   ${_PKG_SILENT}${_PKG_DEBUG}
   if [ `${SH} -c umask` -ne ${DEF_UMASK} ]; then
     ${ECHO_MSG} "${_PKGSRC_IN}>  Warning: your umask is \"`${SH} -c umask`"\".
     ${ECHO_MSG} "If this is not desired, set it to an appropriate value (${DEF_UMASK})"
     ${ECHO_MSG} "and install this package again."
   fi
.endscript
# PLIST AUTOGENERATION -- turned on by the AUTOGENERATE_PLIST variable
.  if defined(AUTOGENERATE_PLIST)
# Prepare the sandbox directory (${WRKDIR}/image)
.    script
.      import AUTOGENERATE_IMAGE AUTOGENERATE_IMAGE_DIRS MKDIR
.      expand
       ${_PKG_SILENT}${_PKG_DEBUG}
.      noexpand
       for d in ${AUTOGENERATE_IMAGE_DIRS}; do
         ${MKDIR} "${AUTOGENERATE_IMAGE}"/"${d}"
       done
.    endscript
.  endif
.  if !defined(AUTOGENERATE_PLIST)
# The old way -- no autogeneration going on.
	${_PKG_SILENT}${_PKG_DEBUG} ${MAKE_INSTALL_TASK} pre-install-script
	${_PKG_SILENT}${_PKG_DEBUG} ${MAKE_INSTALL_TASK} pre-install
	${_PKG_SILENT}${_PKG_DEBUG} ${MAKE_INSTALL_TASK} do-install
	${_PKG_SILENT}${_PKG_DEBUG} ${MAKE_INSTALL_TASK} post-install
	${_PKG_SILENT}${_PKG_DEBUG} ${MAKE_INSTALL_TASK} post-install-script
	@# PLIST must be generated at this late point (instead of
	@# depending on it somewhere earlier), as the
	@# pre/do/post-install aren't run then yet:
	@${_PKG_SILENT}${_PKG_DEBUG} ${MAKE_INSTALL_TASK} ${PLIST:Q}
.    if !defined(NO_PKG_REGISTER)
	${_PKG_SILENT}${_PKG_DEBUG} ${MAKE_INSTALL_TASK} register-pkg
.       if ${GODI_VERIFY_PLIST} == "YES" || ${GODI_VERIFY_PLIST} == "yes"
	     ${_PKG_SILENT}${_PKG_DEBUG} ${MAKE_INSTALL_TASK} verify
.       endif
.    endif # !NO_PKG_REGISTER
.  else
	${_PKG_SILENT}${_PKG_DEBUG} \
          ${ECHO_MSG} "${_PKGSRC_IN}> Installing to image dir"
	${_PKG_SILENT}${_PKG_DEBUG} ${MAKE_INSTALL_TASK_IMG} pre-install-script
	${_PKG_SILENT}${_PKG_DEBUG} ${MAKE_INSTALL_TASK_IMG} pre-install
	${_PKG_SILENT}${_PKG_DEBUG} ${MAKE_INSTALL_TASK_IMG} do-install
	${_PKG_SILENT}${_PKG_DEBUG} ${MAKE_INSTALL_TASK_IMG} post-install
	${_PKG_SILENT}${_PKG_DEBUG} ${MAKE_INSTALL_TASK_IMG} post-install-script
.    if !defined(NO_PKG_REGISTER)
	${_PKG_SILENT}${_PKG_DEBUG} \
          ${MAKE} -C ${.CURDIR:Q} ${MAKEFLAGS} godiva-register-pkg
.       if ${GODI_VERIFY_PLIST} == "YES" || ${GODI_VERIFY_PLIST} == "yes"
	    ${_PKG_SILENT}${_PKG_DEBUG} ${MAKE_INSTALL_TASK} verify
.       endif
.    endif # !NO_PKG_REGISTER
.  endif
.ifdef MESSAGE
	@${ECHO_MSG} "${_PKGSRC_IN}> Please note the following:"
	@${ECHO_MSG} ""
	@${CAT} ${MESSAGE:Q}
	@${ECHO_MSG} ""
.endif
	${_PKG_SILENT}${_PKG_DEBUG}${TOUCH} ${TOUCH_FLAGS} ${INSTALL_COOKIE:Q}


# Fake installation of package so that user can pkg_delete it later.
# Also, make sure that an installed package is recognized correctly in
# accordance to the @pkgdep directive in the packing lists

.if defined(AUTOGENERATE_PLIST)
REGISTER_PREFIX=	${AUTOGENERATE_IMAGE}
.else
REGISTER_PREFIX=	${PREFIX}
.endif


.PHONY: register-pkg
.if !target(register-pkg)
register-pkg: ${PLIST} ${DESCR} ${MESSAGE} prepare-registration
	${_PKG_SILENT}${_PKG_DEBUG}					\
	if [ ! -f ${PLIST:Q} -o ! -f ${DESCR:Q} ]; then			\
		${ERROR_MSG} "** Missing package files for ${PKGNAME} - installation not recorded."; \
		exit 1;							\
	fi
	${_PKG_SILENT}${_PKG_DEBUG}					\
		${ECHO_MSG} "${_PKGSRC_IN}> Registering installation for ${PKGNAME}"; \
		${PKG_PTOOL} \
			-in-dispersed -fixup-symlinks -out-register \
			-set-prefix ${REGISTER_PREFIX:Q} \
			-set-plist ${PLIST:Q} \
			${PTOOL_ARGS}
.endif

.PHONY: godiva-register-pkg
.if !target(godiva-register-pkg)
godiva-register-pkg: ${DESCR} ${MESSAGE} prepare-registration
	${_PKG_SILENT}${_PKG_DEBUG}${ECHO_MSG} "${_PKGSRC_IN}> Installing and registering"
	${_PKG_SILENT}${_PKG_DEBUG}\
	${PKG_PTOOL} -in-dispersed -fixup-symlinks -out-install \
		-set-prefix ${REGISTER_PREFIX:Q} \
		-set-auto-plist \
		${PTOOL_ARGS}
.endif


${FLIST}:
	${_PKG_SILENT}${_PKG_DEBUG}\
	${MAKE} -C ${.CURDIR:Q} ${MAKEFLAGS} file-list-pkg > ${FLIST}
	${_PKG_SILENT}${_PKG_DEBUG} dos2unix -q ${FLIST}

.PHONY: file-list-pkg
.if !target(file-list-pkg)
.  if !defined(AUTOGENERATE_PLIST)
file-list-pkg: ${PLIST} ${DESCR} ${MESSAGE}
	${_PKG_SILENT}${_PKG_DEBUG}\
	     ${PKG_PTOOL} \
		-in-dispersed -out-file-list \
		-set-prefix ${REGISTER_PREFIX:Q} \
		-set-plist ${PLIST:Q} \
		${PTOOL_ARGS_NOBUILD}
.  else
file-list-pkg: ${DESCR} ${MESSAGE}
	${_PKG_SILENT}${_PKG_DEBUG}\
	     ${PKG_PTOOL} \
	        -in-dispersed -fixup-symlinks -out-file-list \
		-set-prefix ${REGISTER_PREFIX:Q} \
		-set-auto-plist \
		${PTOOL_ARGS_NOBUILD}
.  endif
.endif


.PHONY: prepare-registration
prepare-registration: ${FLIST}
.  if defined(FORCE_PKG_REGISTER)
	${_PKG_SILENT}${_PKG_DEBUG} \
          ${RM} -rf ${GODI_DBDIR:Q}/${PKGNAME:Q}
.  endif
	${_PKG_SILENT}${_PKG_DEBUG} \
          ${RM} -f ${BUILD_VERSION_FILE:Q} ${BUILD_INFO_FILE:Q}
#--- Fill BUILD_INFO with details about the used utilities
.  for def in ${BUILD_DEFS}
	@${ECHO} ${def}=${${def}:Q} | \
          ${SED} -e 's|^PATH=[^ 	]*|PATH=...|' >> ${BUILD_INFO_FILE:Q}
.  endfor
	@${ECHO} "CC=${CC}" >> ${BUILD_INFO_FILE:Q}
	@if ${CC} -v 2>&1 | ${GREP} -q '^gcc'; then \
	  ${ECHO} "CC_VERSION=`${CC} -v 2>&1 | ${GREP} '^gcc'`" >> ${BUILD_INFO_FILE:Q}; \
	elif ${CC} -V 2>&1 | ${GREP} -q '^cc'; then \
	  ${ECHO} "CC_VERSION=`${CC} -V 2>&1 | ${GREP} '^cc'`" >> ${BUILD_INFO_FILE:Q}; \
        fi
.  ifdef USE_GMAKE
	@${ECHO} "GMAKE=`${GMAKE} --version | ${GREP} Make`" >> ${BUILD_INFO_FILE:Q}
.  endif
#--- Fill BUILD_VERSION with API checksums etc.
.  script
.    import ${ESSENTIAL_PROGRAMS}
.    import BUILD_VERSION_FILE FLIST DIGEST DISTINFO_FILE PATCH_VERSION_FILE
.    import OPSYS W32PORT MACHINE_ARCH PKGNAME_NOREV REGISTER_PREFIX
.  expand
     ${_PKG_SILENT}${_PKG_DEBUG}
.  noexpand
     { echo "--- Version ---"
       echo "name=${PKGNAME_NOREV}"
       echo "--- Parameters ---"
       echo "OPSYS=$OPSYS"
       echo "W32PORT=$W32PORT"
       echo "MACHINE_ARCH=$MACHINE_ARCH"
       echo "--- Distinfo ---"
       test '!' -f "$DISTINFO_FILE" || $CAT "$DISTINFO_FILE" 
       echo "--- Applied patches ---"
       $CAT "$PATCH_VERSION_FILE"
       echo "--- Interface checksums ---"
       while read file; do
	 full_file="$REGISTER_PREFIX/$file"
	 case "$file" in
	   lib/ocaml/pkg-lib/*/*.cmi)
	     $DIGEST MD5 -strip "$REGISTER_PREFIX/" "$full_file" ;;
	   lib/ocaml/pkg-lib/*/*.cmx)
	     $DIGEST MD5 -strip "$REGISTER_PREFIX/" "$full_file" ;;
	   lib/ocaml/pkg-lib/stublibs/*)
	     echo "exists ($file)" ;;
	   bin/*)
	     echo "exists ($file)" ;;
	   sbin/*)
	     echo "exists ($file)" ;;
	 esac
       done
       echo "--- TIMESTAMP ---"
       rpdir="$(cygpath "$REGISTER_PREFIX")"
       cat "$FLIST" | ( \
	cd "$rpdir" && xargs stat -c "%n %Y" | \
	  md5sum | awk '{print $1}' )
     } < $FLIST > $BUILD_VERSION_FILE
.  endscript


######################################################################
# PACKAGE
######################################################################

PACKAGE_LINKS_FN_IMPORT =
.text
.expand
${ESSENTIAL_PROGRAMS}
PACKAGES
PKGREPOSITORY
PKGREPOSITORYSUBDIR
CATEGORIES
ERROR_MSGS
.endtext


PACKAGE_LINKS_FN=
.text
delete_package_links() {
  # $1: basename of binary package file
  $FIND $PACKAGES -type l -name "$1" | $XARGS $RM -f
}

make_package_links() {
  # $1: basename of binary package file
  delete_package_links "$1"
  for cat in $CATEGORIES; do
    if [ ! -d "$PACKAGES/$cat" ]; then
      $MKDIR "$PACKAGES/$cat"
      if [ $? -ne 0 ]; then
	$ERROR_MSG "=> Can't create directory $PACKAGES/$cat."
	exit 1
      fi
    fi
    $RM -f "$PACKAGES/$cat"/"$1" || exit
    $LN -s "../$PKGREPOSITORYSUBDIR/$1" "$PACKAGES/$cat" || exit
  done
}
.endtext


.PHONY: do-su-package
do-su-package:
	@${ECHO_MSG} "${_PKGSRC_IN}> Packaging ${PKGNAME}"
	${_PKG_SILENT}${_PKG_DEBUG}					\
	realtarget="real-su-package";					\
	action="package";						\
	${_SU_TARGET}


.PHONY: real-su-package
.if !target(real-su-package)
real-su-package:
.	script
.	  import ${PACKAGE_LINKS_FN_IMPORT}
.	  import _PKGSRC_IN PKGNAME ERROR_MSG PKG_PTOOL
.	expand
	${_PKG_SILENT}${_PKG_DEBUG}
	${PACKAGE_LINKS_FN}
.       noexpand
	echo "${_PKGSRC_IN}> Building binary package for $PKGNAME"
	if [ ! -d "$PKGREPOSITORY" ]; then
	  $MKDIR "$PKGREPOSITORY"
	  if [ $? -ne 0 ]; then
	    $ERROR_MSG "=> Can't create directory ${PKGREPOSITORY}."
	    exit 1
	  fi
	fi
	out_name=`${PKG_PTOOL} -in-installed ${PKGNAME} -out-binary ${PKGREPOSITORY} -print-out-name | tr -d '\r'`
	if [ -n "$out_name" ]; then
		base_out_name=`basename "$out_name"`
		make_package_links "$base_out_name" || exit
		${PKG_PTOOL} -in-binary "$out_name" -out-manifest \
			>> ${PKGREPOSITORY}/PKGMANIFEST
	else
	        exit 1
	fi
.	endscript

	${_PKG_SILENT}${_PKG_DEBUG}${TOUCH} ${TOUCH_FLAGS} ${PACKAGE_COOKIE:Q}
.endif


# call as: godi_make PKGMANIFEST_DIR=<dir> to create the PKGMANIFEST
# file in <dir>, listing all the binary packages in this directory.
# Without setting PKGMANIFEST_DIR, this command affects the local
# package repository (build/packages/All).

PKGMANIFEST_DIR ?= $(PKGREPOSITORY)

.PHONY: pkgmanifest
pkgmanifest:
.	script
.         import PKGMANIFEST_DIR PKG_PTOOL
.       noexpand
	rm -f ${PKGMANIFEST_DIR}/PKGMANIFEST
	for f in ${PKGMANIFEST_DIR}/*.tgz; do
	    ${PKG_PTOOL} -in-binary "$f" -out-manifest \
		>> ${PKGMANIFEST_DIR}/PKGMANIFEST
	done
.       endscript

######################################################################
# DEINSTALL
######################################################################

# Deinstall
#
# Special target to remove installation

do-su-deinstall:
	@${ECHO_MSG} "${_PKGSRC_IN}> Deinstalling for ${PKGNAME}"
	${_PKG_SILENT}${_PKG_DEBUG}					\
	realtarget="real-su-deinstall";					\
	action="deinstall";						\
	${_SU_TARGET}

.  ifdef PKG_VERBOSE
real-su-deinstall-flags+=	-v
.  endif

.PHONY: real-su-deinstall
real-su-deinstall:
	${_PKG_SILENT}${_PKG_DEBUG}					\
	found="`${PKG_DEP} \"${PKGWILDCARD}\" 2>/dev/null || ${TRUE}`";	\
	if [ "$$found" != "" ]; then					\
		${ECHO} Running ${PKG_DELETE} ${real-su-deinstall-flags} $$found ; \
		${PKG_DELETE} ${real-su-deinstall-flags} $$found || ${TRUE} ; \
	fi
	@${RM} -f ${INSTALL_COOKIE:Q} ${PACKAGE_COOKIE:Q}

######################################################################
# SKELETONS
######################################################################

.PHONY: fetch
.if !target(fetch)
fetch:
	@cd ${.CURDIR:Q} && ${MAKE} ${MAKEFLAGS} real-fetch
.endif

.PHONY: extract
.if !target(extract)
extract: checksum ${WRKDIR} ${EXTRACT_COOKIE}
.endif

.PHONY: patch
.if !target(patch)
patch: extract ${PATCH_COOKIE}
.endif

.PHONY: configure
.if !target(configure)
configure: patch ${CONFIGURE_COOKIE}
.endif

.PHONY: build
.if !target(build)
build: configure ${BUILD_COOKIE}
.endif

.PHONY: test
.if !target(test)
test: build ${TEST_COOKIE}
.endif

INSTALL_REQS := build
.if ${PKGSRC_RUN_TEST} == "YES" || ${PKGSRC_RUN_TEST} == "yes"
INSTALL_REQS += test
.endif
INSTALL_REQS += ${INSTALL_COOKIE}

.PHONY: install
.if !target(install)
install: ${INSTALL_REQS}
.endif

.PHONY: reinstall
.if !target(reinstall)
reinstall:
	${_PKG_SILENT}${_PKG_DEBUG}${RM} -f ${INSTALL_COOKIE:Q} ${PACKAGE_COOKIE:Q} ${PLIST:Q}
	${_PKG_SILENT}${_PKG_DEBUG}${RM} -rf ${WRKDIR}/image
	${_PKG_SILENT}${_PKG_DEBUG} ${MAKE} ${MAKEFLAGS} install
.endif

.PHONY: package
.if !target(package)
package: install ${PACKAGE_COOKIE}
.endif

.PHONY: deinstall do-su-deinstall
.if !target(deinstall)
deinstall: do-su-deinstall
.endif 

# Build a package but don't check the package cookie
#
.PHONY: repackage
.if !target(repackage)
repackage: pre-repackage package

.PHONY: pre-repackage
pre-repackage:
	${_PKG_SILENT}${_PKG_DEBUG}${RM} -f ${PACKAGE_COOKIE:Q}
.endif

# Build a package but don't check the cookie for installation, also don't
# install package cookie
#
.PHONY: package-noinstall
.if !target(package-noinstall)
package-noinstall:
	${_PKG_SILENT}${_PKG_DEBUG}cd ${.CURDIR:Q} && ${MAKE} ${MAKEFLAGS} PACKAGE_NOINSTALL=yes real-package
.endif


# Set cookies

${EXTRACT_COOKIE}:
	${_PKG_SILENT}${_PKG_DEBUG}cd ${.CURDIR:Q} && ${MAKE} ${MAKEFLAGS} real-extract DEPENDS_TARGET=${DEPENDS_TARGET}

${PATCH_COOKIE}:
	${_PKG_SILENT}${_PKG_DEBUG}cd ${.CURDIR:Q} && ${MAKE} ${MAKEFLAGS} real-patch

${CONFIGURE_COOKIE}:
	${_PKG_SILENT}${_PKG_DEBUG}cd ${.CURDIR:Q} && ${MAKE} ${MAKEFLAGS} real-configure

${BUILD_COOKIE}:
	${_PKG_SILENT}${_PKG_DEBUG}cd ${.CURDIR:Q} && ${MAKE} ${MAKEFLAGS} real-build

${TEST_COOKIE}:
	${_PKG_SILENT}${_PKG_DEBUG}cd ${.CURDIR:Q} && ${MAKE} ${MAKEFLAGS} real-test

${INSTALL_COOKIE}:
	${_PKG_SILENT}${_PKG_DEBUG}cd ${.CURDIR:Q} && ${MAKE} ${MAKEFLAGS} real-install

${PACKAGE_COOKIE}:
	${_PKG_SILENT}${_PKG_DEBUG}cd ${.CURDIR:Q} && ${MAKE} ${MAKEFLAGS} real-package


.PHONY: extract-message patch-message configure-message
.PHONY: build-message test-message
extract-message:
	@${ECHO_MSG} "${_PKGSRC_IN}> Extracting for ${PKGNAME}"
patch-message:
	@${ECHO_MSG} "${_PKGSRC_IN}> Patching for ${PKGNAME}"
configure-message:
	@${ECHO_MSG} "${_PKGSRC_IN}> Configuring for ${PKGNAME}"
build-message:
	@${ECHO_MSG} "${_PKGSRC_IN}> Building for ${PKGNAME}"
test-message:
	@${ECHO_MSG} "${_PKGSRC_IN}> Testing for ${PKGNAME}"

.PHONY: extract-cookie patch-cookie configure-cookie
.PHONY: build-cookie test-cookie
extract-cookie:
	${_PKG_SILENT}${_PKG_DEBUG}${ECHO} ${PKGNAME:Q} >> ${EXTRACT_COOKIE:Q}
patch-cookie:
	${_PKG_SILENT}${_PKG_DEBUG} ${TOUCH} ${TOUCH_FLAGS} ${PATCH_COOKIE:Q}
configure-cookie:
	${_PKG_SILENT}${_PKG_DEBUG} ${TOUCH} ${TOUCH_FLAGS} ${CONFIGURE_COOKIE:Q}
build-cookie:
	${_PKG_SILENT}${_PKG_DEBUG} ${TOUCH} ${TOUCH_FLAGS} ${BUILD_COOKIE:Q}
test-cookie:
	${_PKG_SILENT}${_PKG_DEBUG} ${TOUCH} ${TOUCH_FLAGS} ${TEST_COOKIE:Q}

.ORDER: pre-fetch do-fetch post-fetch
.ORDER: extract-message pre-extract do-extract post-extract extract-cookie
.ORDER: patch-message pre-patch do-patch post-patch patch-cookie
.ORDER: configure-message pre-configure pre-configure-override do-configure post-configure configure-cookie
.ORDER: build-message pre-build do-build post-build build-cookie
.ORDER: test-message pre-test do-test post-test test-cookie

# Please note that the order of the following targets is important, and
# should not be modified (.ORDER is not recognised by make(1) in a serial
# make i.e. without -j n)
.PHONY: real-fetch real-extract real-patch real-configure
.PHONY: real-build real-test real-install real-package
real-fetch: pre-fetch do-fetch post-fetch
real-extract: extract-message pre-extract do-extract post-extract extract-cookie
real-patch: patch-message pre-patch do-patch post-patch patch-cookie
real-configure: configure-message pre-configure pre-configure-override do-configure post-configure configure-cookie
real-build: build-message pre-build do-build post-build build-cookie
real-test: test-message pre-test do-test post-test test-cookie
real-install: do-su-install
real-package: do-su-package

###
# Empty pre-* and post-* targets

.for name in fetch extract patch configure build test install-script install package

.  if !target(pre-${name})
pre-${name}:
	@${DO_NADA}
.  endif

.  if !target(post-${name})
post-${name}:
	@${DO_NADA}
.  endif

.endfor

######################################################################
# UTILITIES
######################################################################

# The 'info' target can be used to display information about a package.
.PHONY: info
info:
	${_PKG_SILENT}${_PKG_DEBUG}${PKG_INFO} "${PKGWILDCARD}"

# The 'list' target can be used to list the files installed by a package.
.PHONY: list
list:
	${_PKG_SILENT}${_PKG_DEBUG}${PKG_INFO} -list "${PKGWILDCARD}"


# Cleaning up

.PHONY: pre-clean
.if !target(pre-clean)
pre-clean:
	@${DO_NADA}
.endif

.PHONY: clean
.if !target(clean)
clean: pre-clean
	@${ECHO_MSG} "${_PKGSRC_IN}> Cleaning for ${PKGNAME}"
	${_PKG_SILENT}${_PKG_DEBUG}					\
	if [ -d ${WRKDIR:Q} ]; then					\
		if [ -w ${WRKDIR:Q} ]; then				\
			${RM} -rf ${WRKDIR:Q};				\
		else							\
			${ECHO_MSG} "${_PKGSRC_IN}> ${WRKDIR} not writable, skipping"; \
		fi;							\
	fi
.  ifdef WRKOBJDIR
	-${_PKG_SILENT}${_PKG_DEBUG}					\
	${RMDIR} ${BUILD_DIR:Q} 2>/dev/null;				\
	${RM} -f ${WRKDIR_BASENAME:Q}
.  endif
.endif


.PHONY: pre-distclean
.if !target(pre-distclean)
pre-distclean:
	@${DO_NADA}
.endif


.PHONY: cleandir
.if !target(cleandir)
cleandir: clean
.endif


.PHONY: distclean
.if !target(distclean)
distclean: pre-distclean clean
	${_PKG_SILENT}${ECHO_MSG} "${_PKGSRC_IN}> Dist cleaning for ${PKGNAME}"
	${_PKG_SILENT}${_PKG_DEBUG}if [ -d ${DISTDIR:Q} ]; then	\
		cd ${DISTDIR:Q} &&					\
		${TEST} -z "${DISTFILES}" || ${RM} -f ${DISTFILES:Q};	\
	fi
.  if defined(DIST_SUBDIR) && exists(DIST_SUBDIR)
	-${_PKG_SILENT}${_PKG_DEBUG}${RMDIR} ${DISTDIR:Q}
.  endif
.endif

# Print installed files:
# BUG: This method relies on the correctness of file time stamps. This is
# problematic for ocamlfind-based installations, because this tool does
# not update the mtime for a number of reasons. However, ctime is updated.
# So one workaround is to consider the ctime, too. However, not every
# version of "find" supports ctime. This is tested here.

.if target(print-installed)
HAVE_FIND_WITH_CTIME!=                                         \
  if ${FIND} / -prune -cnewer / >/dev/null 2>/dev/null; then   \
      ${ECHO} "yes";                                           \
  else                                                         \
      ${ECHO} "";                                              \
  fi
.endif

.PHONY: print-installed
print-installed:
	${_PKG_SILENT}${_PKG_DEBUG}\
	cd ${PREFIX:Q} && \
	${FIND} . \! -type d \( -newer ${EXTRACT_COOKIE:Q} ${HAVE_FIND_WITH_CTIME:S|^yes$|-o -cnewer ${EXTRACT_COOKIE:Q}|} \) | \
	${GREP} -v '^./build/' | \
	${GREP} -v '^${GODI_DBDIR:C|/+|/|g:S|/$||:S|^${PREFIX:C|/+|/|g:S|/$||}/|./|}/' | \
	${SED} -e 's|^./||'


# This rule prints a suggestion for the PLIST.godi file. Note that it
# never generates @findlib lines.

print-PLIST.godi:
	@${MAKE} PKGNAME='_dummy_' PKGVERSION='_dummy_' PKGLOCALEDIR='_dummy_' \
	    print-PLIST | {                                           \
		while read f; do                                      \
		    case "$$f" in                                     \
			\@*)                                          \
			    echo "$$f" ;;                             \
			*)                                            \
			    case `basename $$f` in                    \
				dll*.so|dll*.owner)                   \
					echo "@ocamldll $$f" ;;       \
				lib*.a)                               \
					echo "$$f" ;;                 \
				*.a|*.cmx|*.cmxa|*.o|*.opt|ocamlopt)  \
					echo "@ocamlopt $$f" ;;       \
				*)                                    \
					echo "$$f" ;;                 \
			    esac                                      \
			    ;;                                        \
		    esac;                                             \
		done;                                                 \
	    } |                                                       \
	    sed 's:lib/godi/${V_TM}:lib/godi/$${V_TM}:'


######################################################################
# Checksumming utilities
######################################################################

.PHONY: makesum
.if !target(makesum)
makesum: fetch
.script
.expand
	${_PKG_SILENT}${_PKG_DEBUG}
	newfile=${DISTINFO_FILE}.$$$$
	${ECHO} '$$GODI$$' > $$newfile
	${ECHO} "" >> $$newfile
	cd ${DISTDIR}
	for sumfile in "" ${CKSUMFILES}; do
		if [ "X$$sumfile" = "X" ]; then continue; fi
		${DIGEST} ${DIGEST_ALGORITHM} < "${DISTPREFIX}$$sumfile" |
		    ${AWK} -v pfile="${DISTPREFIX_IN_DISTINFO}$$sumfile" \
	                '{ print "${DIGEST_ALGORITHM} (" pfile ") = " $$1 }' \
		    >> $$newfile
		${WC} -c < "${DISTPREFIX}$$sumfile" | 
	            ${AWK} -v pfile="${DISTPREFIX_IN_DISTINFO}$$sumfile" \
	                '{ print "Size (" pfile ") = " $$1 " bytes" }' \
                    >> $$newfile
	done
	for ignore in "" ${IGNOREFILES}; do
		if [ "X$$ignore" = "X" ]; then continue; fi
		${ECHO} "${DIGEST_ALGORITHM} (${DISTPREFIX_IN_DISTINFO}$$ignore) = IGNORE" >> $$newfile
	done
	if [ -f ${DISTINFO_FILE} ]; then
		${AWK} '$$2 ~ /\(patch-.*\)/ { print $$0 }
                        $$2 ~ /\(file-.*\)/ { print $$0 }' < ${DISTINFO_FILE} >> $$newfile
	fi
	if ${CMP} -s $$newfile ${DISTINFO_FILE}; then
		${RM} -f $$newfile
		${ECHO_MSG} "=> distinfo: distfiles part unchanged."
	else
		${MV} $$newfile ${DISTINFO_FILE}
	fi
.endscript
.endif

.if !target(makepatchsum)
makepatchsum mps:
.script
.expand
	${_PKG_SILENT}${_PKG_DEBUG}
	newfile=${DISTINFO_FILE}.$$$$
	if [ -f ${DISTINFO_FILE} ]; then
		${AWK} '$$2 !~ /\(patch-.*\)/ { print $$0 }' < ${DISTINFO_FILE} | ${AWK} '$$2 !~ /\(file-.*\)/ { print $$0 }' >> $$newfile
	else
		${ECHO} '$$GODI$$' >$$newfile
		${ECHO} "" >> $$newfile
	fi
	if [ -d ${PATCHDIR} ]; then
		(  cd ${PATCHDIR}
		   for sumfile in "" patch-* file-*; do
			if [ "X$$sumfile" = "X" ]; then continue; fi
			if [ "X$$sumfile" = "Xpatch-*" ]; then continue; fi
			if [ "X$$sumfile" = "Xfile-*" ]; then continue; fi
			${DIGEST} ${DIGEST_ALGORITHM} $$sumfile >> $$newfile
		   done
                )
	fi
	if ${CMP} -s $$newfile ${DISTINFO_FILE}; then
		${RM} -f $$newfile
		${ECHO_MSG} "=> distinfo: patches part unchanged."
	else
		${MV} $$newfile ${DISTINFO_FILE}
	fi
.endscript
.endif

# This target is done by invoking a sub-make so that DISTINFO_FILE gets
# re-evaluated after the "makepatchsum" target is made. This can be
# made into:
#makedistinfo mdi: makepatchsum makesum
# once a combined distinfo file exists for all packages
.if !target(makedistinfo)
makedistinfo mdi distinfo: makepatchsum
	${_PKG_SILENT}${_PKG_DEBUG}${MAKE} makesum
.endif


######################################################################
# Helpers
######################################################################

.if !target(show-shlib-type)
# Show the shared lib type being built: one of ELF, a.out, dylib, or none
# GODI: This piece of code is kept inside this file because it is unclear
# whether it is still needed.
.PHONY: show-shlib-type
show-shlib-type:
.  if exists(/usr/lib/libc.dylib)
	${_PKG_SILENT}${_PKG_DEBUG}					\
	${ECHO} "dylib"
.  else
	${_PKG_SILENT}${_PKG_DEBUG}					\
	cd ${WRKDIR} &&							\
	sotype=none;							\
	if [ "X${MKPIC}" != "Xno" -a "X${NOPIC}" = "X" ]; then		\
		${ECHO} "int main() { exit(0); }" > a.$$$$.c;		\
		${CC} ${CFLAGS} a.$$$$.c -o a.$$$$.out;			\
		case `${FILE_CMD} a.$$$$.out` in			\
		*ELF*dynamically*)					\
			sotype=ELF ;;					\
		*shared*library*)					\
			sotype="a.out" ;;				\
		*dynamically*)						\
			sotype="a.out" ;;				\
		esac;							\
	fi;								\
	${ECHO} "$$sotype";						\
	${RM} -f a.$$$$.c a.$$$$.out
.  endif # libc.dylib
.endif

# convenience target, to display make variables from command line
# i.e. "make show-var VARNAME=var", will print var's value
.PHONY: show-var
show-var:
	@${ECHO} ${${VARNAME}:Q}

# enhanced version of target above, to display multiple variables
.PHONY: show-vars
show-vars:
.for VARNAME in ${VARNAMES}
	@${ECHO} ${${VARNAME}:Q}
.endfor

######################################################################
# Helpers for PLIST verification                                     #
######################################################################

.if ${GODI_VERIFY_PLIST} == "YES" || ${GODI_VERIFY_PLIST} == "yes"
CP_FILE=${LOCALBASE}/build/files_checkpoint
CP_REG_FILE=${LOCALBASE}/build/files_registered
CP_ACT_FILE=${LOCALBASE}/build/files_actual  

CHECKPOINT_DIRS = ${LOCALBASE_DIRS} ${CHECKPOINT_DIRS_ADDITIONS}

.PHONY: checkpoint
checkpoint:
	@${ECHO_MSG} "${_PKGSRC_IN}> Checkpointing"
	${_PKG_SILENT}${_PKG_DEBUG}                                    \
	cd ${LOCALBASE}; \
	find ${CHECKPOINT_DIRS} -type f -o -type l 2>/dev/null | \
	    LC_ALL=C sort >${CP_FILE}

.PHONY: verify
verify:
	@${ECHO_MSG} "${_PKGSRC_IN}> Verifying PLIST"
	${_PKG_SILENT}${_PKG_DEBUG}                                    \
	{ godi_info -file-list ${PKGNAME}; \
          cat ${CP_FILE}; \
        }  | \
            LC_ALL=C sort | uniq >${CP_REG_FILE}
	${_PKG_SILENT}${_PKG_DEBUG}                                    \
	cd ${LOCALBASE}; \
        find ${CHECKPOINT_DIRS} -type f -o -type l 2>/dev/null | \
            LC_ALL=C sort | uniq >${CP_ACT_FILE}
	${_PKG_SILENT}${_PKG_DEBUG}                                    \
	if ! diff -b -B ${CP_REG_FILE} ${CP_ACT_FILE} >/dev/null 2>/dev/null; then \
            echo "### PLIST incorrect: ${PKGNAME}"; \
            echo "### Unregistered files:"; \
	    if comm --version >/dev/null 2>/dev/null; then \
                LC_ALL=C comm -2 -3 ${CP_ACT_FILE} ${CP_REG_FILE}; \
            else \
		LC_ALL=C diff ${CP_ACT_FILE} ${CP_REG_FILE}; \
	    fi \
        fi
.endif


################################################################
# Release

.PHONY: release
release:
.script
.import AWK GODI_API GODI_BUILD_SVN SECTIONS GODI_CONSOLE GODI_USER \
	PKGBASE _PKG_SILENT _PKG_DEBUG SED
.expand
	${_PKG_SILENT}${_PKG_DEBUG}
.noexpand
	repo_root="$(LC_ALL=C svn info . | ${AWK} '/Repository Root/ { print $3 }')"
	if [ "X$repo_root" != "X${GODI_BUILD_SVN}" ]; then
	    echo "Error: Bad repository root";
	    exit 1
	fi
	repo_url="$(LC_ALL=C svn info . | ${AWK} '/URL/ { print $2 }')"
	svnpath="$($AWK -v repo_root="$repo_root/" -v repo_url="$repo_url" \
                   'BEGIN {print substr(repo_url, length(repo_root)+1)}')"
	svnrev="$(LC_ALL=C svnversion -c .)"
	case "$svnrev" in
	    *[MSP]*)
		echo "Error: svn working copy is not clean"
		exit 1 ;;
	    exported)
		echo "Error: not in a svn working copy"
		exit 1 ;;
	    *:*)
		svnrev="$(echo "$svnrev" | $SED -e 's/.*://')"
		;;
	esac
	echo "SVN path: $svnpath"
	echo "SVN rev:  $svnrev"
	if [ -z "$SECTIONS" ]; then
	    echo "The variable SECTIONS is not set, but this variable is required."
	    echo "SECTIONS is expected to contain the GODI section identifiers"
	    echo "(like 3.12 or 4.00) where this package is to be released."
	    echo "Set SECTIONS on the command-line, or add it to Makefile."
	fi
	for section in $SECTIONS; do
	    ${GODI_CONSOLE} wtool \
		-site "$GODI_API" \
		-user "$GODI_USER" \
	        -api function=release section="$section" package="$PKGBASE" \
                     svnpath="$svnpath" svnrev="$svnrev"
	done
.endscript


################################################################
# Export

.PHONY: export-build
export-build: export-build-tgz export-write-manifest

.PHONY: export-profile
export-profile: export-write-profile

.PHONY: export-package
export-package: export-write-package

.PHONY: export-seed
export-seed: export-write-seed

.ifmake export-build || export-profile || export-package || export-seed

EXPORT_BUILDFILES ?= ${EXPORT_DESTDIR}/buildfiles
EXPORT_DISTFILES ?= ${EXPORT_DESTDIR}/distfiles
EXPORT_PROFILES ?= ${EXPORT_DESTDIR}/profiles
EXPORT_PACKAGES ?= ${EXPORT_DESTDIR}/packages
EXPORT_SEEDS ?= ${EXPORT_DESTDIR}/seeds

EXPORT_SECTION ?= ${GODI_SECTION}
# TODO: better is to extract the section from EXPORT_PROFILE_NAME

.ifndef EXPORT_SOURCE
.BEGIN::
	@echo "Missing variable: EXPORT_SOURCE"
	@false
.endif

EXPORT_PROPS = \
    -pkg-name ${PKGNAME:QQ} \
    -pkg-source ${EXPORT_SOURCE:QQ} \
    -pkg-comment ${COMMENT:QQ} \
    -pkg-descr-file DESCR \
    -pkg-maintainer ${MAINTAINER:QQ} \
    -pkg-homepage ${HOMEPAGE:QQ} \
    -pkg-section ${EXPORT_SECTION:QQ}
.for _df in ${DISTFILES}
    EXPORT_PROPS += -pkg-distfile ${_df:QQ}
.endfor

EXPORT_PROFILE_NAME ?= ${GODI_PROFILE}


.PHONY: export-build-tgz
export-build-tgz:
	test -f Makefile
	test -f DESCR
	test -f distinfo
#       test -f PLIST -o -f PLIST.godi
.script
.expand
	${_PKG_SILENT}${_PKG_DEBUG}
	set -e
	${MKDIR} "${EXPORT_BUILDFILES}"
	${MKDIR} "${EXPORT_DISTFILES}"
	cd ..
	{ ${FIND} ${PKGBASE} \
	      '!' -name ".*" '!' -name "*~" '!' \
              '(' -name "work" -prune ')' \
              -type f
	} | ${PAX} -w -z -x tar -d \
	        -f ${EXPORT_BUILDFILES:Q}/${PKGNAME:Q}.build.tgz
	echo Wrote ${EXPORT_BUILDFILES:Q}/${PKGNAME:Q}.build.tgz
	for df in ${DISTFILES:QL}; do
	    ${CP} ${DISTDIR:Q}/"$$df" ${EXPORT_DISTFILES:Q}/
	    echo Wrote ${EXPORT_DISTFILES}/"$$df"
	done
.endscript

.PHONY: export-write-manifest
export-write-manifest:
.script
.expand
	${_PKG_SILENT}${_PKG_DEBUG}
	${TOUCH} "${EXPORT_BUILDFILES}/packages_all.ini"
	${PKG_TOOLS_BIN}/godi_console initool \
		-o "${EXPORT_BUILDFILES}/packages_all.ini.new" \
		"${EXPORT_BUILDFILES}/packages_all.ini" \
		-add ${EXPORT_PROPS}
	${MV} \
		"${EXPORT_BUILDFILES}/packages_all.ini.new" \
		"${EXPORT_BUILDFILES}/packages_all.ini"
	${PKG_TOOLS_BIN}/godi_console initool \
		-o "${EXPORT_BUILDFILES}/packages.ini" \
		"${EXPORT_BUILDFILES}/packages_all.ini" \
		-newest
	${PKG_TOOLS_BIN}/godi_console initool \
		-o "${EXPORT_BUILDFILES}/available.new" \
		"${EXPORT_BUILDFILES}/packages_all.ini" \
		-newest -rocketboost
.endscript

.PHONY: export-write-profile
export-write-profile:
.script
.expand
	${_PKG_SILENT}${_PKG_DEBUG}
	${MKDIR} -p "${EXPORT_PROFILES}"
	${PKG_TOOLS_BIN}/godi_console rprofile \
		-site "${EXPORT_PROFILES}" \
		-put "${EXPORT_PROFILE_NAME}"
	echo Wrote "${EXPORT_PROFILES}/${EXPORT_PROFILE_NAME}.profile.tgz"
.endscript


.PHONY: export-write-package
export-write-package:
.script
.expand
	${_PKG_SILENT}${_PKG_DEBUG}
	${MKDIR} "${EXPORT_PACKAGES}"
	out_name=`${PKG_PTOOL} -in-installed ${PKGNAME} \
		-out-binary ${EXPORT_PACKAGES:QQ} -print-out-name | tr -d '\r'`
	if [ -n "$$out_name" ]; then
		${PKG_PTOOL} -in-binary "$$out_name" -out-manifest \
			>> ${EXPORT_PACKAGES:QQ}/PKGMANIFEST
	else
	        exit 1
	fi
.endscript

.PHONY: export-write-seed
export-write-seed:
.script
.expand
	${_PKG_SILENT}${_PKG_DEBUG}
	set -e
	${MKDIR} "${EXPORT_SEEDS}"
	gt_name="$$(${PKG_DEP} -names 'godi-tools-*')"
	name="seed-using-$${gt_name}.tar.gz"
	cd ${LOCALBASE:Q}
	${MKDIR} "build/export.tmp"
	${TOUCH} "build/export.tmp/empty1"
	${TOUCH} "build/export.tmp/empty2"
	sed -e 's/^GODI_PROFILE[ \t]*=[ \t]*.*$$/GODI_PROFILE=${EXPORT_PROFILE_NAME}/' \
	    etc/godi.conf > build/export.tmp/godi.conf
	echo "GODI_SECTION=${EXPORT_SECTION}" > build/export.tmp/profile.conf
	dollar="$$"
	cat <<EOF > build/export.tmp/README
This tar.gz file contains a binary seed for a GODI deployment.
Generally, this seed will run on any OS/OS version that is compatible
with the OS where this seed file was created.

You MUST unpack this seed file to the directory ${LOCALBASE}:

$${dollar} mkdir ${LOCALBASE}
$${dollar} cd ${LOCALBASE}
$${dollar} tar xzf <directory>/$$name

It will not work correctly in other directories, because it is not
relocatable.

Then, add ${LOCALBASE}/bin and 
${LOCALBASE}/sbin to PATH:

$${dollar} export PATH=${LOCALBASE}/bin:${LOCALBASE}/sbin:$${dollar}PATH

After that, you can run godi_console to start GODI. Note that a seed
file is empty, so you will need to update the list of packages, or
select binary packages.

OS/machine properties:

OPSYS:           ${OPSYS}
OS_VERSION:      ${OS_VERSION}
MACHINE:         ${MACHINE}
MACHINE_ARCH:    ${MACHINE_ARCH}
EOF
	{ echo "bin"
	  echo "bin/godi_confdir"
	  echo "build"
	  echo "build/buildfiles"
	  echo "build/distfiles"
	  echo "build/packages"
	  echo "build/packages/All"
	  echo "build/export.tmp/empty1" # empty PKGMANIFEST
	  echo "build/profile-patches"
	  echo "build/profiles"
	  ${FIND} "build/profiles/${EXPORT_PROFILE_NAME}" -type f
	  echo "build/export.tmp/empty2" # empty available.ini
	  echo "db/$${gt_name}"
	  for f in "db/$${gt_name}"/+*; do echo "$$f"; done
	  echo "doc"
	  echo "doc/GODI/bootstrap"
	  for f in "doc/GODI/bootstrap"/*; do echo "$$f"; done
	  echo "build/export.tmp/godi.conf"
	  echo "build/export.tmp/profile.conf"
	  echo "lib"
	  echo "build/export.tmp/README"
	  ${PKG_TOOLS_BIN}/godi_console info -file-list godi-tools
	} | ${PAX} -w -z -x tar -d \
	        -s ':build/export.tmp/empty1:build/packages/All/PKGMANIFEST:' \
	        -s ':build/export.tmp/empty2:build/available.ini:' \
		-s ':build/export.tmp/godi.conf:etc/godi.conf:' \
		-s ':build/export.tmp/profile.conf:etc/profile.conf:' \
		-s ':build/export.tmp/README:README:' \
		-f "${EXPORT_SEEDS}/$$name"
	echo "Wrote ${EXPORT_SEEDS}/$$name"
.endscript

.endif
# .ifmake

######################################################################
.endif # PACKAGE_SCANNING
######################################################################
