#!/bin/bash

set -e

godi_dir="$(dirname "$(godi_confdir)")"
godi_conf="${godi_dir}/etc/godi.conf"
godi_sbin_dir="${godi_dir}/sbin"
godi_sbin_tmp_dir="${godi_dir}/sbin.tmp"
godi_console="${godi_sbin_dir}/godi_console.exe"

if [ ! -d "$godi_sbin_dir" ]; then
    echo "godi sbin dir missing" >&2
    exit 1
fi
if [ ! -x "$godi_console" ] ; then
    echo "godi_console missing" >&2
    exit 1
fi
if [ ! -f "$godi_conf" ]; then
    echo "godi.conf not found" >&2
    exit 1
fi

OBJDUMP="$(godi_make -f "${godi_conf}" -V MINGW_TOOL_PREFIX || echo '')"
OBJDUMP="${OBJDUMP}objdump"

declare -A mymap

oldIFS=$IFS
IFS='
'

all_dlls="$($OBJDUMP -p "$godi_console" | grep 'DLL Name:' | awk '{print $3}')"
while read dll ; do
    if [ -z "$dll" ] || [ ! -f "${godi_sbin_dir}/${dll}" ] ; then
        continue
    fi
    mymap[$dll]=1
done <<EOF
"$all_dlls"
EOF

mkdir -p "$godi_sbin_tmp_dir"
cp -p "${godi_sbin_dir}/bsdtar.exe" "${godi_sbin_dir}/bsdcpio.exe" "$godi_console" "$godi_sbin_tmp_dir"

new_found=1
while [ $new_found -eq 1 ]; do
    new_found=0
    for dll in ${!mymap[@]} ; do
        value=${mymap[$dll]}
        if [ $value -eq 1 ]; then
            mymap[$dll]=2
            cp -p "${godi_sbin_dir}/${dll}" "${godi_sbin_tmp_dir}"
            all_dlls="$($OBJDUMP -p "${godi_sbin_dir}/${dll}" | grep 'DLL Name:' | awk '{print $3}')"
            while read ndll ; do
                if [ -z "$ndll" ] || [ "a${mymap[$ndll]}" = "a1" ] || \
                    [ "a${mymap[$ndll]}" = "a2" ] || [ ! -f "${godi_sbin_dir}/${ndll}" ] ; then
                    continue
                fi
                mymap[$ndll]=1
                new_found=1
            done <<EOF
$all_dlls
EOF
        fi
    done
done
IFS=$oldIFS

# temporary workaround. Does objdump really report all dlls ?!
cd "${godi_sbin_dir}"
for f in *.dll ; do
    [ ! -f "$f" ] && continue
    if [ ! -f "${godi_sbin_tmp_dir}/${f}" ]; then
        cp -p "$f" "${godi_sbin_tmp_dir}"
    fi
done

exit 0
