#! /bin/bash

set -e
path="$(cygpath "$1")"
godiroot="$(readlink -f "$path")"
winroot="$(cygpath -m "$godiroot")"

portable_remove_from_path() {
    local NPATH remaining dir to_remove do_stop
    to_remove=$1
    do_stop=2
    case ${PATH+set} in
        set)
            remaining=$PATH
            while [ $do_stop -gt 0 ]; do
                dir=${remaining%%:*}
                if [ "$dir" != "$to_remove" ]; then
                    if [ -z "$NPATH" ]; then
                        NPATH=$dir
                    else
                        NPATH="${NPATH}:${dir}"
                    fi
                fi

                remaining=${remaining#*:}
                case $remaining in
                    *:*)
                        continue ;;
                    *)
                        do_stop=$(( $do_stop - 1 ))
                        ;;
                esac
            done
            PATH=$NPATH
            ;;
        *)
            ;;
    esac
}
portable_remove_from_path "${godiroot}/bin"
portable_remove_from_path "${godiroot}/sbin"

PATH="${godiroot}/bin:${godiroot}/sbin:${PATH}"
export PATH

tmpfile="$(mktemp)"

trap "rm -f \"$tmpfile\"" EXIT

confdat="${godiroot}/etc/godi.conf"

cp -p "$confdat" "$tmpfile"

cygwinroot="$(cygpath -m "/")"

cygwinwordsize="$(/bin/uname -m)"
case "$cygwinwordsize" in
    x86_64)
        cygwinwordsize=64 ;;
    *i[3-9]86*)
        cygwinwordsize=32 ;;
    *_64*)
        cygwinwordsize=64 ;;
    *)
        cygwinwordsize="$(/bin/arch)"
        case "$cygwinwordsize" in
            *64*)
                cygwinwordsize=64 ;;
            *)
                cygwinwordsize=32 ;;
        esac
esac

sed -e "s|^\s*LOCALBASE_NATIVE\s*=.*$|LOCALBASE_NATIVE = $winroot |" \
    -e "s|^\s*GODI_USER\s*=.*$|GODI_USER = anonymous |" \
    -e "s|^\s*CYGWIN_ROOT\s*=.*$|CYGWIN_ROOT = $cygwinroot |" \
    -e "s|^\s*CYGWIN_WORDSIZE\s*=.*$|CYGWIN_WORDSIZE = $cygwinwordsize |" \
    "$tmpfile"  > "$confdat"

dos2unix "${confdat}"

"${godiroot}/lib/godi/winconfig.sh" --first-run

GODI_PREFIX=${godiroot}
export GODI_PREFIX

cd "${godiroot}/db"
for f in \
    godi-ocaml-4*/+INSTALL \
    godi-camlp4-*/+INSTALL \
    godi-findlib-1*/+INSTALL \
    godi-flexdll-0*/+INSTALL \
    apps-ocaml-top-1*/+INSTALL \
    apps-ocamltop-win-*/+INSTALL ; do

    [ ! -f "$f" ] && continue
    bash "$f" x 'POST-INSTALL' || exit 1
done
