#!/bin/bash

set -e

if [ -z "$USER" ]; then
    USER=
fi
param_one="$1"
set -u

APP_DIR="$(/bin/dirname $0)"
APP_DIR="$(cd "$APP_DIR"; pwd)"
APP_DIR="$(/bin/readlink -f "$APP_DIR")"

REL_DIR="$(/bin/basename "$(/bin/readlink -f "${APP_DIR}" )" )"

cd "$APP_DIR"
case "$REL_DIR" in
    wodi64)
        bit=64
        ;;
    *)
        bit=32
        ;;
esac

wodidir="/opt/wodi${bit}"
bsdtar="${APP_DIR}/sbin/bsdtar.exe"

if [ -e "$wodidir" ]; then
    echo "${wodidir} already exists." >&2
    echo "Please remove this installation of WODI first!" >&2
    echo "Or just rename ${wodir} temporary." >&2
    exit 1
fi

do_abort=0

install_message(){
    echo "please install ${1} with cygwin's setup.exe and than call this script again" >&2
    do_abort=1
}

checkp(){
    if [ ! -x "$1" ]; then
        install_message "$2"
    fi
}

if [ $bit -eq 32 ]; then
    checkp /usr/bin/i686-w64-mingw32-gcc "mingw64-i686-gcc-core"
else
    checkp /usr/bin/x86_64-w64-mingw32-gcc "mingw64-x86_64-gcc-core"
fi

checkp /usr/bin/cpio "cpio"
checkp /usr/bin/patch "patch"
checkp /usr/bin/make "make"

checkp /usr/bin/ash "dash"
checkp /usr/bin/awk "gawk"
checkp /usr/bin/bash "bash"
checkp /usr/bin/bzip2 "bzip2"
checkp /usr/bin/cat "coreutils(cat)"
checkp /usr/bin/chmod "coreutils(chmod)"
checkp /usr/bin/chown "coreutils(chown)"
checkp /usr/bin/cmp "diffutils(cmp)"
checkp /usr/bin/cp "coreutils(cp)"
checkp /usr/bin/cut "coreutils(cut)"
checkp /usr/bin/cygpath "cygwin(cygpath)"
checkp /usr/bin/dash "dash"
checkp /usr/bin/diff "diffutils"
checkp /usr/bin/dos2unix "dos2unix"
checkp /usr/bin/echo "coreutils(echo)"
checkp /usr/bin/env "coreutils(env)"
checkp /usr/bin/false "coreutils(false)"
checkp /usr/bin/file "file"
checkp /usr/bin/find "findutils"
checkp /usr/bin/getent "getent"
checkp /usr/bin/grep "grep"
checkp /usr/bin/gzip "gzip"
checkp /usr/bin/head "coreutils(head)"
checkp /usr/bin/ln "coreutils(ln)"
checkp /usr/bin/mintty "mintty"
checkp /usr/bin/mkdir "coreutils(mkdir)"
checkp /usr/bin/mktemp "coreutils(mktemp)"
checkp /usr/bin/mount "cygwin(mount)"
checkp /usr/bin/mv "coreutils(mv)"
checkp /usr/bin/printf "coreutils(printf)"
checkp /usr/bin/readlink "coreutils(readlink)"
checkp /usr/bin/sed "sed"
checkp /usr/bin/sh "bash"
checkp /usr/bin/sleep "coreutils(sleep)"
checkp /usr/bin/sort "coreutils(sort)"
checkp /usr/bin/stat "coreutils(stat)"
checkp /usr/bin/tail "coreutils(tail)"
checkp /usr/bin/tar "tar"
checkp /usr/bin/touch "coreutils(touch)"
checkp /usr/bin/tr "coreutils(tr)"
checkp /usr/bin/true "coreutils(true)"
checkp /usr/bin/uname "coreutils(uname)"
checkp /usr/bin/wc "coreutils(wc)"
checkp /usr/bin/xz "xz"

if [ $do_abort -eq 1 ]; then
    exit 1
fi

#report errors of this script
wodistep=0
wodiinstallversion=2
wkernel="$(/bin/uname -s 2>/dev/null || echo unknown)"
warch="$(/bin/uname -m 2>/dev/null || echo unknown)"
bsdtmpdir="$(/bin/mktemp -d)"
mclean(){
    # local url="http://ml.ignorelist.com/wodi-install${wodiinstallversion},${wkernel},${warch},${wodistep}.txt"
    # local got=0
    # if which wget >/dev/null 2>&1 ; then
    #     wget --timeout=5 --spider --quiet -O - "$url" >/dev/null 2>&1 || true
    #     got=1
    # elif which curl >/dev/null 2>&1 ; then
    #     curl --connect-timeout 5 --max-time 6 -s -I "$url" >/dev/null 2>&1 || true
    #     got=1
    # fi
    if [ -n "$bsdtmpdir" ]; then
        # if [ $got -eq 0 ]; then
        #     GODI_CONFDIR="${APP_DIR}/etc"
        #     export GODI_CONFDIR
        #     /bin/chmod 0755 "${APP_DIR}/sbin/"*.[Dd][Ll][Ll] >/dev/null 2>&1 || true
        #     "${APP_DIR}/sbin/godi_console.exe" wget -O "${bsdtmpdir}/p${wodistep}.txt" \
        #         "http://ml.ignorelist.com/wodi-install${wodiinstallversion},${wkernel},${warch},${wodistep}.txt" >/dev/null 2>&1 || true
        # fi
        /bin/rm -rf "$bsdtmpdir"
        bsdtmpdir=''
    fi
    if [ "$wodistep" != "ok" ]; then
        echo "installation failed :-(" >&2
    fi
}
trap mclean EXIT

opt_created=0
if [ ! -e "/opt" ]; then
    opt_created=1
fi

/bin/mkdir -p "${wodidir}"
wodistep=1
SID=""

default_secure(){
    dir=$1
    bdir=$2
    curdir=$3
    echo "failed to secure $dir" >&2
    echo "Set access rights for ${bdir} manually (if necessary)" >&2
    echo "If the installations fails, please remove $dir manually and run this script" >&2
    echo "again with '-i' as first parameter: " >&2
    echo "  bash ${0} -i" >&2
    /bin/chmod 0755 "${bdir}" || true
    cd "$curdir"
    return
}
secure_dir(){
    local dir=$1
    local secured=0
    local set_manually=0
    if [ -z "$SID" ]; then
        wodistep="a${wodistep}"
        SID="$(/bin/getent passwd "$USER" | /usr/bin/awk -F':' '{print $5}' | /usr/bin/awk -F',' '{print $NF}' || echo '')"
        wodistep="b${wodistep}"
    fi
    if [ -z "$SID" ]; then
        echo "Warning: Couldn't determine USER SID" >&2
        echo "Set access rights for ${dir} manually (if necessary)" >&2
        /bin/chmod 0755 "${dir}" || true
        return
    fi

    wodistep="c${wodistep}"
    local pdir="$(/bin/dirname "$dir")"
    local bdir="$(/bin/basename "$dir")"
    local curdir="$(pwd)"
    wodistep="d${wodistep}"
    cd "$pdir"

    if ! icacls.exe "${bdir}" /reset >/dev/null 2>&1 ; then
        default_secure "$dir" "$bdir" "$curdir"
        return
    fi
    # old icacls version don't support /inheritance
    if ! icacls.exe "${bdir}" /inheritance:r  >/dev/null 2>&1 ; then
        echo "Warning: old icacls version / icacls failed!" >&2
        default_secure "$dir" "$bdir" "$curdir"
        return
    fi
    if icacls.exe "${bdir}" /grant "*${SID}:(F)" >/dev/null 2>&1 && \
        icacls.exe "${bdir}" /grant '*S-1-1-0:(CI)(OI)(IO)(RX)' >/dev/null 2>&1 && \
        icacls.exe "${bdir}" /grant '*S-1-3-0:(OI)(CI)(IO)(F)' >/dev/null 2>&1 && \
        icacls.exe "${bdir}" /grant '*S-1-3-1:(OI)(CI)(IO)(RX)' >/dev/null 2>&1 && \
        /bin/chmod 0755 "${bdir}"; then
        cd "$curdir"
        return
    fi
    default_secure "$dir" "$bdir" "$curdir"
    return
}

if [[ "$param_one" != "-i" ]]; then
    wodistep=2
    if [ -n "$USER" ] && which icacls.exe >/dev/null 2>&1; then
        wodistep=3
        secure_dir "${wodidir}"
        if [ $opt_created -eq 1 ]; then
            wodistep=4
            secure_dir /opt
        fi
        wodistep=5
    else
        wodistep=6
        /bin/chmod 0755 /opt
        wodistep=7
        /bin/chmod 0755 "${wodidir}"
        wodistep=8
        echo "Warning: icacls not found or $USER not set" >&2
        echo "Set access rights for ${wodidir} manually (if necessary)" >&2
    fi
fi

wodistep=9
/bin/cp -p "$bsdtar" "${bsdtmpdir}/bsdtar.exe"
wodistep=10
"${bsdtmpdir}/bsdtar.exe" -cf- bin build db doc etc gui include lib man sbin share var | ( cd "${wodidir}" && "${bsdtmpdir}/bsdtar.exe" -xf- )
wodistep=11

PATH="${wodidir}/bin":"${wodidir}/sbin":$PATH
export PATH

#"${wodidir}/gui/share/godi-gui/relocate.sh" "$wodidir"
#if ! /bin/bash "${wodidir}/relocate.sh" "$wodidir" ; then
if ! "${wodidir}/gui/share/godi-gui/relocate.sh" "$wodidir" ; then
    echo "relocate error!" >&2
    exit 1
fi
wodistep=12

"${wodidir}/lib/godi/winconfig.sh" --first-run
wodistep=13

if [ ! -f "${wodidir}/sbin/bsdtar.exe" ]; then
    wodistep=14
    "${wodidir}/bin/godi_install.exe" -m 0755 "$bsdtar" "${wodidir}/sbin/bsdtar.exe"
fi
wodistep=15
/bin/cat - <<EOF
Wodi is now installed!

Links to the cygwin shell and the package management tool have been
created in cygwin's root folder.

Start a new shell (e.g. bash --login) in order to use WODI and try one
of the following customizations (a new shell is necessary in order to
run one of the following commands):

Create link in the start menu by calling:
${wodidir}/lib/godi/winconfig.sh --add-startmenu-folder

Add links to the desktop:
${wodidir}/lib/godi/winconfig.sh --add-to-desktop

Set environment variables like OCAMLLIB outside cygwin:
${wodidir}/lib/godi/winconfig.sh --global-environment

Add ocaml's folder to %PATH%
${wodidir}/lib/godi/winconfig.sh --add-ocaml-path

Add cygwin's /usr/bin folder to %PATH%:
${wodidir}/lib/godi/winconfig.sh --add-cygwin-path

EOF
wodistep=ok
